"""前話情報抽出×動的プロンプト生成統合テスト"""

from pathlib import Path
from tempfile import TemporaryDirectory
from unittest.mock import Mock

import pytest

from noveler.application.use_cases.previous_episode_analysis_use_case import (
    EnhancedPromptGenerationRequest,
    PreviousEpisodeAnalysisRequest,
    PreviousEpisodeAnalysisUseCase,
)
from noveler.domain.entities.chapter_plot import ChapterPlot
from noveler.domain.entities.previous_episode_context import (
    CharacterState,
    PreviousEpisodeContext,
    StoryProgressionState,
    TechnicalLearningState,
)
from noveler.domain.services.contextual_inference_engine import ContextualInferenceEngine
from noveler.domain.services.enhanced_staged_prompt_generation_service import EnhancedStagedPromptGenerationService
from noveler.domain.services.previous_episode_extraction_service import PreviousEpisodeExtractionService
from noveler.domain.value_objects.episode_number import EpisodeNumber
from noveler.presentation.shared.shared_utilities import get_common_path_service


@pytest.mark.spec("SPEC-ENHANCED-001")
class TestEnhancedPreviousEpisodeIntegration:
    """前話情報抽出と動的プロンプト生成の統合テスト"""

    def setup_method(self):
        """テスト前準備"""
        get_common_path_service()
        self.use_case = PreviousEpisodeAnalysisUseCase()
        self.temp_dir = TemporaryDirectory()
        self.project_root = Path(self.temp_dir.name)

        # テスト用ディレクトリ構造作成
        self._setup_test_directory_structure()

    def teardown_method(self):
        """テスト後処理"""
        self.temp_dir.cleanup()

    def _setup_test_directory_structure(self) -> None:
        """テスト用ディレクトリ構造セットアップ"""
        # 原稿ディレクトリ作成
        path_service = get_common_path_service()
        manuscript_dir = self.project_root / str(path_service.get_manuscript_dir())
        manuscript_dir.mkdir(parents=True, exist_ok=True)

        # プロット関連ディレクトリ作成
        plot_dir = self.project_root / str(path_service.get_plots_dir()) / "章別プロット"
        plot_dir.mkdir(parents=True, exist_ok=True)

        # テスト用原稿ファイル作成
        self._create_test_manuscript(manuscript_dir)

    def _create_test_manuscript(self, manuscript_dir: Path) -> None:
        """テスト用原稿ファイル作成"""
        manuscript_content = """# 第006話 ペアプログラミング基礎

## 導入部
直人は朝から緊張していた。今日はあすかと一緒にペアプログラミングの練習をする日だった。

「おはよう、直人！」
あすかが教室に入ってきた。彼女はいつも通り元気だった。

## 展開部
実習室で、先生がペアプログラミングの基礎を説明し始めた。

「ペアプログラミングとは、2人で1つのプログラムを作成する手法です」

直人はメモを取りながら真面目に聞いていた。あすかは既に理解しているようで、時々頷いていた。

実際にペアプログラミングを始めてみると、思っていたより難しかった。

「ここはどうする？」
「こうしてみよう」

二人は協力して魔法のプログラムを作成した。シンクロ率が徐々に向上していく。

## クライマックス
複雑なアルゴリズムの実装で困難に直面した。

「うまくいかないな」
直人は困っていた。

「大丈夫、一緒に考えよう」
あすかが励ました。

二人で協力することで、ついに難しい問題を解決できた。

## 解決部
協調魔法の基礎を理解し、シンクロ率も大幅に向上した。

「今日は本当に勉強になった」
直人は満足していた。

「次回はもっと高度なことにチャレンジしよう」
あすかが提案した。

次回の協力が楽しみになっていた。まだペアプログラミングの基礎段階だが、将来的にはより高度な協調開発を目指したい。
"""
        manuscript_file = manuscript_dir / "第006話_ペアプログラミング基礎.md"
        manuscript_file.write_text(manuscript_content, encoding="utf-8")

    @pytest.mark.asyncio
    async def test_previous_episode_analysis_comprehensive(self):
        """前話情報分析の包括的テスト"""
        # Given: 第7話の前話分析リクエスト
        request = PreviousEpisodeAnalysisRequest(
            episode_number=7,
            project_root=self.project_root,
            analysis_depth="comprehensive",
            include_inference_details=True,
        )

        # When: 前話情報分析実行
        response = await self.use_case.analyze_previous_episode(request)

        # Then: 分析成功と詳細情報取得確認
        assert response.success, f"Analysis failed: {response.error_message}"
        assert response.episode_number == 7
        assert response.previous_context is not None
        assert response.dynamic_context is not None

        # 前話コンテキストの詳細確認
        prev_context = response.previous_context
        assert prev_context.current_episode_number.value == 7
        assert prev_context.previous_episode_number.value == 6
        assert prev_context.has_sufficient_context()

        # キャラクター状態抽出確認
        assert len(prev_context.character_states) > 0
        assert "直人" in prev_context.character_states
        assert "あすか" in prev_context.character_states

        # 直人の状態確認
        naoto_state = prev_context.get_character_state("直人")
        assert naoto_state is not None
        assert naoto_state.emotional_state != "平静"  # 何らかの感情が抽出されているはず

        # あすかの状態確認
        asuka_state = prev_context.get_character_state("あすか")
        assert asuka_state is not None

        # ストーリー進行状況確認
        story_prog = prev_context.story_progression
        assert story_prog.story_momentum in ["low", "normal", "high", "climactic"]

        # 技術学習状況確認
        tech_learning = prev_context.technical_learning
        assert tech_learning.current_learning_focus != ""  # 学習フォーカス抽出確認

        # 動的コンテキストの確認
        dynamic_context = response.dynamic_context
        assert dynamic_context.story_phase in ["introduction", "development", "climax", "resolution"]
        assert dynamic_context.character_growth_stage in ["beginner", "learning", "practicing", "competent", "expert"]
        assert dynamic_context.technical_complexity_level in ["basic", "intermediate", "advanced", "expert"]

        # 推論結果確認
        assert len(dynamic_context.inferences) > 0
        high_confidence_inferences = dynamic_context.get_high_confidence_inferences()
        assert len(high_confidence_inferences) >= 0  # 0以上の高信頼度推論

        # 分析サマリー確認
        assert "前話情報分析結果" in response.analysis_summary
        assert "動的推論結果" in response.analysis_summary

        # 推論詳細確認（リクエストで要求されている）
        assert len(response.inference_insights) >= 0

        # 推奨事項確認
        assert len(response.recommendations) > 0

    @pytest.mark.asyncio
    async def test_enhanced_prompt_generation_with_previous_context(self):
        """前話コンテキストを活用した高度化プロンプト生成テスト"""
        # Given: 章別プロット情報準備
        chapter_plot = self._create_test_chapter_plot()

        # Given: 高度化プロンプト生成リクエスト
        request = EnhancedPromptGenerationRequest(
            episode_number=7,
            project_root=self.project_root,
            chapter_plot=chapter_plot,
            target_stage=3,  # シーン肉付け段階
            base_context={"project_name": "test_novel"},
        )

        # When: 高度化プロンプト生成実行
        response = await self.use_case.generate_enhanced_prompt(request)

        # Then: 生成成功と高度化機能確認
        assert response.success, f"Enhanced prompt generation failed: {response.error_message}"
        assert response.episode_number == 7
        assert response.target_stage == 3
        assert response.generated_prompt != ""
        assert response.quality_score > 0.0

        # 前話コンテキスト活用確認
        assert response.previous_context_used  # 前話情報が活用されている

        # 動的調整適用確認
        assert response.dynamic_adjustments_applied  # 動的調整が適用されている

        # 高度化詳細情報確認
        enhancement_details = response.enhancement_details
        assert "story_phase" in enhancement_details
        assert "character_growth_stage" in enhancement_details
        assert "technical_complexity_level" in enhancement_details
        assert enhancement_details["emotional_focus_count"] >= 0
        assert enhancement_details["adaptive_elements_count"] >= 0

        # 生成されたプロンプトの内容確認
        generated_prompt = response.generated_prompt
        assert "Stage 3" in generated_prompt  # 段階情報が含まれている

        # 適応的調整の確認（プロンプトに特定の指示が含まれているか）
        if response.enhancement_details.get("emotional_focus_count", 0) > 2:
            # 多数の感情フォーカス領域がある場合、感情描写に関する指示があるはず
            assert any(keyword in generated_prompt.lower() for keyword in ["感情", "emotional", "描写", "内面"])

    @pytest.mark.spec("SPEC-ENHANCED_PREVIOUS_EPISODE_INTEGRATION-PREVIOUS_EPISODE_EXT")
    def test_previous_episode_extraction_service_integration(self):
        """前話情報抽出サービスの統合テスト"""
        # Given: エピソード番号
        episode_number = EpisodeNumber(7)

        # Given: 前話情報抽出サービス
        extraction_service = PreviousEpisodeExtractionService()

        # When: 前話情報抽出実行
        context = extraction_service.extract_previous_episode_context(episode_number, self.project_root)

        # Then: 抽出結果確認
        assert context is not None
        assert context.current_episode_number.value == 7
        assert context.previous_episode_number.value == 6
        assert context.source_manuscript_path is not None
        assert context.source_manuscript_path.exists()

        # キャラクター抽出確認
        assert len(context.character_states) >= 2  # 直人とあすか

        # 技術要素抽出確認
        tech_learning = context.technical_learning
        assert tech_learning.current_learning_focus != ""

        # 感情の流れ抽出確認
        assert len(context.emotional_flow) >= 0

        # YAML変換確認
        yaml_dict = context.to_yaml_dict()
        assert "current_episode_number" in yaml_dict
        assert "character_states" in yaml_dict
        assert "story_progression" in yaml_dict
        assert "technical_learning" in yaml_dict

    @pytest.mark.spec("SPEC-ENHANCED_PREVIOUS_EPISODE_INTEGRATION-CONTEXTUAL_INFERENCE")
    def test_contextual_inference_engine_integration(self):
        """コンテキスト推論エンジンの統合テスト"""
        # Given: エピソード番号と前話コンテキスト
        episode_number = EpisodeNumber(7)

        # 前話コンテキストのモック作成
        previous_context = self._create_mock_previous_context(episode_number)

        # Given: 推論エンジン
        inference_engine = ContextualInferenceEngine()

        # When: 動的コンテキスト生成
        dynamic_context = inference_engine.generate_dynamic_context(
            episode_number,
            None,  # 章別プロット無し
            previous_context,
        )

        # Then: 推論結果確認
        assert dynamic_context is not None
        assert dynamic_context.episode_number.value == 7
        assert dynamic_context.story_phase in ["introduction", "development", "climax", "resolution"]
        assert dynamic_context.character_growth_stage in ["beginner", "learning", "practicing", "competent", "expert"]
        assert dynamic_context.technical_complexity_level in ["basic", "intermediate", "advanced", "expert"]

        # 推論詳細確認
        assert len(dynamic_context.inferences) >= 4  # 少なくとも4つの推論（フェーズ、成長、複雑度、感情、適応）

        # 高信頼度推論確認
        high_confidence = dynamic_context.get_high_confidence_inferences()
        assert len(high_confidence) >= 0

        # YAML変換確認
        yaml_dict = dynamic_context.to_yaml_dict()
        assert "episode_number" in yaml_dict
        assert "story_phase" in yaml_dict
        assert "character_growth_stage" in yaml_dict
        assert "inferences" in yaml_dict

    @pytest.mark.asyncio
    async def test_error_handling_integration(self):
        """エラーハンドリング統合テスト"""
        # Given: 存在しないプロジェクトパス
        invalid_path = Path("/nonexistent/path")

        # When: 無効なパスでの前話分析
        request = PreviousEpisodeAnalysisRequest(episode_number=7, project_root=invalid_path)

        response = await self.use_case.analyze_previous_episode(request)

        # Then: フォールバックが機能し成功扱いになることを確認
        assert response.success is True
        assert response.error_message is None
        assert response.previous_context is not None
        assert response.recommendations

        # When: 無効なパスでの高度化プロンプト生成
        enhanced_request = EnhancedPromptGenerationRequest(episode_number=7, project_root=invalid_path)

        enhanced_response = await self.use_case.generate_enhanced_prompt(enhanced_request)

        # Then: フォールバックプロンプトが生成されることを確認
        assert enhanced_response.success is True
        assert enhanced_response.error_message is None
        assert enhanced_response.generated_prompt
        safety = getattr(enhanced_response, "safety_measures", None)
        if safety:
            assert safety.get("fallback_mode") is True
            assert safety.get("safe_mode") is True

    @pytest.mark.asyncio
    async def test_first_episode_special_handling(self):
        """第1話特別処理テスト"""
        # Given: 第1話の分析リクエスト
        request = PreviousEpisodeAnalysisRequest(episode_number=1, project_root=self.project_root)

        # When: 第1話分析実行
        response = await self.use_case.analyze_previous_episode(request)

        # Then: 第1話特別処理確認
        assert response.success
        assert "第1話のため前話情報はありません" in response.analysis_summary
        assert len(response.recommendations) > 0
        assert any("導入" in rec or "世界観" in rec for rec in response.recommendations)

    def _create_test_chapter_plot(self) -> ChapterPlot:
        """テスト用章別プロット作成"""
        mock_plot = Mock(spec=ChapterPlot)
        mock_plot.get_episode_info.return_value = {
            "episode_number": 7,
            "title": "ペアプログラミング応用編",
            "summary": "より高度な協調開発に挑戦",
            "estimated_length": 6500,
            "key_themes": ["協力", "技術向上", "挑戦"],
            "character_focus": ["直人", "あすか"],
            "technical_elements": ["ペアプログラミング", "協調開発", "高度アルゴリズム"],
        }
        return mock_plot

    def _create_mock_previous_context(self, episode_number: EpisodeNumber) -> PreviousEpisodeContext:
        """モック前話コンテキスト作成"""
        context = PreviousEpisodeContext(episode_number)

        # キャラクター状態追加
        naoto_state = CharacterState(
            character_name="直人",
            emotional_state="成長の喜び",
            current_relationships={"あすか": "協力的"},
            character_development_stage="実践段階",
            key_attributes=["真面目", "協調性"],
        )

        context.add_character_state(naoto_state)

        asuka_state = CharacterState(
            character_name="あすか",
            emotional_state="満足",
            current_relationships={"直人": "友好的"},
            character_development_stage="指導段階",
            key_attributes=["積極的", "協調性"],
        )

        context.add_character_state(asuka_state)

        # ストーリー進行状況設定
        story_progression = StoryProgressionState(
            main_plot_developments=["ペアプログラミング基礎習得", "協調魔法実践"],
            subplot_progressions={"技術学習": "進展", "人間関係": "進展"},
            resolved_conflicts=["初期の緊張"],
            active_foreshadowing=["次回はより高度な挑戦"],
            story_momentum="normal",
        )

        context.update_story_progression(story_progression)

        # 技術学習状況設定
        tech_learning = TechnicalLearningState(
            mastered_concepts=["ペアプログラミング基礎", "協調魔法入門"],
            current_learning_focus="ペアプログラミング",
            difficulty_level="intermediate",
            practical_applications=["魔法プログラム作成", "シンクロ率向上"],
            next_learning_targets=["高度な協調開発"],
        )

        context.update_technical_learning(tech_learning)

        # 感情の流れ追加
        context.add_emotional_flow_element("緊張 → 理解")
        context.add_emotional_flow_element("困惑 → 協力")
        context.add_emotional_flow_element("達成感 → 満足")

        # 未解決要素追加
        context.add_unresolved_element("より高度なアルゴリズム習得")

        # シーン継続性追加
        context.add_scene_continuity_note("前話の主要場所: 実習室で")
        context.add_scene_continuity_note("継続要素: 次回協力予定")

        return context


@pytest.mark.spec("SPEC-ENHANCED-002")
class TestEnhancedStagedPromptGenerationService:
    """高度化段階的プロンプト生成サービステスト"""

    def setup_method(self):
        """テスト前準備"""
        self.temp_dir = TemporaryDirectory()
        self.project_root = Path(self.temp_dir.name)

    def teardown_method(self):
        """テスト後処理"""
        self.temp_dir.cleanup()

    @pytest.mark.spec("SPEC-ENHANCED_PREVIOUS_EPISODE_INTEGRATION-ENHANCED_SERVICE_INI")
    def test_enhanced_service_initialization(self):
        """高度化サービス初期化テスト"""
        # Given: モックリポジトリとバリデータ
        mock_template_repo = Mock()
        mock_quality_validator = Mock()

        # When: 高度化サービス作成
        service = EnhancedStagedPromptGenerationService(mock_template_repo, mock_quality_validator, self.project_root)

        # Then: 正常初期化確認
        assert service is not None
        assert service._project_root == self.project_root
        assert service._previous_extractor is not None
        assert service._inference_engine is not None


if __name__ == "__main__":
    pytest.main([__file__, "-v", "--tb=short"])



# File: src/noveler/application/use_cases/project_init_use_case.py
# Purpose: Orchestrate project initialization workflow
# Context: Application layer use case implementing transaction pattern

"""
ProjectInitUseCase - プロジェクト初期化ワークフロー

This use case orchestrates the project initialization workflow by coordinating
domain services and infrastructure repositories.

Responsibilities:
- Validate preconditions before initialization
- Coordinate directory creation and file writing
- Apply templates with proper parameters
- Ensure transaction guarantees (rollback on failure)

Design Principles:
- Single Responsibility: Project initialization workflow only
- Dependency Inversion: Depends on abstractions (IPathService, etc.)
- Transaction Pattern: Rollback capability for failed operations
"""

from dataclasses import dataclass
from pathlib import Path

from noveler.domain.services.project_initializer_service import (
    ProjectConfig,
    ProjectInitializerService,
)
from noveler.infrastructure.repositories.template_repository import (
    TemplateRepository,
)


@dataclass
class ProjectInitRequest:
    """
    Request for project initialization.

    Attributes:
        project_name: Project name to create
        project_root: Parent directory where project will be created
        template_name: Optional template override (default: "base")
        genre: Optional genre override
        pen_name: Optional pen name override
    """

    project_name: str
    project_root: Path
    template_name: str = "base"
    genre: str | None = None
    pen_name: str | None = None


@dataclass
class ProjectInitResult:
    """
    Result of project initialization.

    Attributes:
        success: Whether initialization succeeded
        project_path: Path to created project directory
        created_dirs: List of created directory paths
        config_path: Path to project configuration file
        error_message: Error message if failed
    """

    success: bool
    project_path: Path | None = None
    created_dirs: list[str] = None
    config_path: Path | None = None
    error_message: str | None = None

    def __post_init__(self):
        """Initialize empty lists."""
        if self.created_dirs is None:
            self.created_dirs = []


class ProjectInitializationError(Exception):
    """Raised when project initialization fails."""



class ProjectInitUseCase:
    """
    Use case for project initialization workflow.

    This use case coordinates domain services and infrastructure repositories
    to create a new project with standard structure and configuration.
    """

    def __init__(
        self,
        initializer_service: ProjectInitializerService,
        template_repository: TemplateRepository,
    ):
        """
        Initialize use case with dependencies.

        Args:
            initializer_service: Domain service for project initialization logic
            template_repository: Repository for template access
        """
        self._initializer = initializer_service
        self._templates = template_repository

    def execute(self, request: ProjectInitRequest) -> ProjectInitResult:
        """
        Execute project initialization workflow.

        This method orchestrates the complete initialization workflow:
        1. Validate preconditions
        2. Create directory structure
        3. Render and write configuration files
        4. Render and write README and .gitignore

        Transaction Guarantee:
            If any step fails, all created files/directories are rolled back.

        Args:
            request: Project initialization request

        Returns:
            ProjectInitResult with success status and created paths

        Examples:
            >>> use_case = ProjectInitUseCase(service, repo)
            >>> request = ProjectInitRequest(
            ...     project_name="08_時空の図書館",
            ...     project_root=Path("/novels")
            ... )
            >>> result = use_case.execute(request)
            >>> result.success
            True
        """
        created_dirs = []
        created_files = []

        try:
            # Step 1: Validate preconditions
            validation_result = self.validate_preconditions(request)
            if not validation_result.success:
                return validation_result

            # Step 2: Generate configuration
            project_path = request.project_root / request.project_name
            config = self._initializer.generate_config(
                project_name=request.project_name,
                project_root=project_path,
                genre=request.genre,
                pen_name=request.pen_name,
            )

            # Step 3: Create project root directory
            project_path.mkdir(parents=True, exist_ok=True)
            created_dirs.append(str(project_path))

            # Step 4: Create standard directory structure
            dir_structure = self._initializer.generate_directory_structure()
            for dir_path in dir_structure.directories:
                full_path = project_path / dir_path
                full_path.mkdir(parents=True, exist_ok=True)
                created_dirs.append(str(full_path))

            # Step 5: Render and write project configuration
            config_content = self._render_config_template(config)
            config_path = project_path / "プロジェクト設定.yaml"
            config_path.write_text(config_content, encoding="utf-8")
            created_files.append(str(config_path))

            # Step 6: Render and write README
            readme_content = self._render_readme_template(config)
            readme_path = project_path / "README.md"
            readme_path.write_text(readme_content, encoding="utf-8")
            created_files.append(str(readme_path))

            # Step 7: Render and write .gitignore
            gitignore_content = self._render_gitignore_template()
            gitignore_path = project_path / ".gitignore"
            gitignore_path.write_text(gitignore_content, encoding="utf-8")
            created_files.append(str(gitignore_path))

            return ProjectInitResult(
                success=True,
                project_path=project_path,
                created_dirs=created_dirs,
                config_path=config_path,
            )

        except Exception as e:
            # Rollback: Remove created files and directories
            self._rollback(created_files, created_dirs)

            return ProjectInitResult(
                success=False,
                error_message=f"Project initialization failed: {e!s}",
            )

    def validate_preconditions(
        self, request: ProjectInitRequest
    ) -> ProjectInitResult:
        """
        Validate preconditions before initialization.

        Checks:
        - Project name is valid
        - Parent directory exists and is writable
        - Project directory does not already exist (or warn if it does)
        - Required templates exist

        Args:
            request: Project initialization request

        Returns:
            ProjectInitResult with success=True if valid, success=False otherwise
        """
        # Validate project name
        is_valid, error = self._initializer.validate_project_name(
            request.project_name
        )
        if not is_valid:
            return ProjectInitResult(success=False, error_message=error)

        # Check parent directory exists
        if not request.project_root.exists():
            return ProjectInitResult(
                success=False,
                error_message=f"Parent directory does not exist: {request.project_root}",
            )

        # Check parent directory is writable
        if not request.project_root.is_dir():
            return ProjectInitResult(
                success=False,
                error_message=f"Not a directory: {request.project_root}",
            )

        # Warn if project directory already exists
        project_path = request.project_root / request.project_name
        if project_path.exists():
            # Note: In production, this would prompt user for confirmation
            # For now, we allow overwrite (existing files won't be deleted)
            pass

        # Validate templates exist
        required_templates = [
            f"project/{request.template_name}/プロジェクト設定.yaml.j2",
            f"project/{request.template_name}/README.md.j2",
            f"project/{request.template_name}/.gitignore.j2",
        ]

        for template_name in required_templates:
            if not self._templates.validate_template_exists(template_name):
                return ProjectInitResult(
                    success=False,
                    error_message=f"Required template not found: {template_name}",
                )

        return ProjectInitResult(success=True)

    def _render_config_template(self, config: ProjectConfig) -> str:
        """Render project configuration template."""
        template_name = "project/base/プロジェクト設定.yaml.j2"
        return self._templates.render_template_by_name(
            template_name,
            {
                "project_root": str(config.project_root),
                "title": config.title,
                "genre": config.genre,
                "status": config.status,
                "created_date": config.created_date,
                "pen_name": config.pen_name,
            },
        )

    def _render_readme_template(self, config: ProjectConfig) -> str:
        """Render README.md template."""
        template_name = "project/base/README.md.j2"
        return self._templates.render_template_by_name(
            template_name,
            {
                "title": config.title,
                "created_date": config.created_date,
                "project_name": config.project_name,
            },
        )

    def _render_gitignore_template(self) -> str:
        """Render .gitignore template."""
        template_name = "project/base/.gitignore.j2"
        return self._templates.render_template_by_name(template_name, {})

    def _rollback(self, created_files: list[str], created_dirs: list[str]):
        """
        Rollback created files and directories on failure.

        Args:
            created_files: List of file paths to remove
            created_dirs: List of directory paths to remove
        """
        # Remove files first
        for file_path_str in created_files:
            try:
                file_path = Path(file_path_str)
                if file_path.exists():
                    file_path.unlink()
            except Exception:
                # Best effort: continue even if deletion fails
                pass

        # Remove directories (in reverse order to handle nested dirs)
        for dir_path_str in reversed(created_dirs):
            try:
                dir_path = Path(dir_path_str)
                if dir_path.exists() and dir_path.is_dir():
                    # Only remove if empty
                    if not any(dir_path.iterdir()):
                        dir_path.rmdir()
            except Exception:
                # Best effort: continue even if deletion fails
                pass



#!/usr/bin/env python3
"""プロット分析エンティティ

プロット分析のライフサイクルを管理する
"""

from dataclasses import dataclass, field
from datetime import datetime
from uuid import uuid4

from noveler.domain.ai_integration.value_objects.analysis_result import AnalysisResult
from noveler.domain.value_objects.project_time import ProjectTimezone, project_now

# JSTタイムゾーン
JST = ProjectTimezone.jst().timezone


@dataclass
class PlotAnalysis:
    """プロット分析エンティティ

    分析の識別子、対象ファイル、分析日時、結果を管理
    """

    id: str
    plot_file_path: str
    created_at: datetime | None = None
    analyzed_at: datetime | None = None
    result: AnalysisResult | None = field(default=None)

    def __post_init__(self) -> None:
        """Ensure temporal metadata is populated for legacy call sites."""
        if self.created_at is None:
            self.created_at = project_now().datetime
        if self.analyzed_at is None:
            self.analyzed_at = self.created_at

    def __hash__(self) -> int:
        """idと分析対象ファイルパスによるハッシュ値を生成"""
        return hash((self.id, self.plot_file_path))

    @classmethod
    def create(cls, plot_file_path: str) -> "PlotAnalysis":
        """新しいプロット分析を作成"""
        now = project_now().datetime
        return cls(
            id=f"analysis_{uuid4().hex[:8]}",
            plot_file_path=plot_file_path,
            created_at=now,
            analyzed_at=now,
            result=None,
        )

    def set_result(self, result: AnalysisResult) -> None:
        """分析結果を設定"""
        if self.result is not None:
            msg = "分析結果は既に設定されています"
            raise ValueError(msg)
        self.result = result

    def is_analyzed(self) -> bool:
        """分析が完了しているかどうか"""
        return self.result is not None

    def get_elapsed_time(self) -> float:
        """分析開始からの経過時間(秒)"""
        reference = self.analyzed_at or project_now().datetime
        return (project_now().datetime - reference).total_seconds()

    def __eq__(self, other: object) -> bool:
        """IDによる同一性判定"""
        if not isinstance(other, PlotAnalysis):
            return False
        return self.id == other.id



#!/usr/bin/env python3
"""ユーザーガイダンス機能のドメイン層テスト
TDD+DDD統合開発による実装

ビジネスルールをテストコードで表現し、
ユーザビリティ向上のビジネスロジックを検証


仕様書: SPEC-UNIT-TEST
"""

import sys
import unittest
from pathlib import Path

import pytest
import yaml

# スクリプトのルートディレクトリをパスに追加
script_dir = Path(__file__).resolve().parent
project_root = script_dir.parent
sys.path.insert(0, str(project_root))

from noveler.domain.entities.error_context import ErrorContext, ErrorSeverity
from noveler.domain.entities.progress_report import NextAction, ProgressReport, ProgressStatus
from noveler.domain.entities.user_guidance import GuidanceStep, GuidanceType, UserGuidance
from noveler.domain.repositories.plot_progress_repository import PlotProgressRepository
from noveler.domain.services.plot_progress_service import PlotProgressService
from noveler.domain.services.smart_error_handler_service import SmartErrorHandlerService
from noveler.domain.services.user_guidance_service import UserGuidanceService
from noveler.domain.value_objects.time_estimation import TimeEstimation
from noveler.domain.value_objects.workflow_stage_type import WorkflowStageType
from noveler.presentation.shared.shared_utilities import get_common_path_service


class TestGuidanceStep(unittest.TestCase):
    """ガイダンスステップのテスト"""

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CREATE_GUIDANCE_STEP")
    def test_create_guidance_step(self) -> None:
        """ガイダンスステップを作成できる"""
        get_common_path_service()
        step = GuidanceStep(
            step_number=1,
            title="企画書作成",
            description="作品のコンセプトを決めましょう",
            command="cp template.yaml project.yaml",
            time_estimation=TimeEstimation.from_minutes(30),
            is_completed=False,
        )

        assert step.step_number == 1
        assert step.title == "企画書作成"
        assert not step.is_completed
        assert step.time_estimation.in_minutes() == 30

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-MARK_STEP_AS_COMPLET")
    def test_mark_step_as_completed(self) -> None:
        """ステップを完了状態にできる"""
        step = GuidanceStep(
            step_number=1,
            title="テスト",
            description="テスト",
            command="test",
            time_estimation=TimeEstimation.from_minutes(10),
        )

        assert not step.is_completed
        step.mark_as_completed()
        assert step.is_completed

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-DETERMINE_STEP_EXECU")
    def test_determine_step_executability(self) -> None:
        """ステップの実行可能性を判定できる"""
        step = GuidanceStep(
            step_number=1,
            title="テスト",
            description="テスト",
            command="test",
            time_estimation=TimeEstimation.from_minutes(10),
            prerequisites=["file1.yaml", "file2.yaml"],
        )

        # 前提条件が満たされていない場合
        assert not step.can_execute(existing_files=[])

        # 前提条件が満たされている場合
        assert step.can_execute(existing_files=["file1.yaml", "file2.yaml"])


class TestUserGuidance(unittest.TestCase):
    """ユーザーガイダンスエンティティのテスト"""

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CREATE_USER_GUIDANCE")
    def test_create_user_guidance(self) -> None:
        """ユーザーガイダンスを作成できる"""
        steps = [
            GuidanceStep(1, "ステップ1", "説明1", "cmd1", TimeEstimation.from_minutes(10)),
            GuidanceStep(2, "ステップ2", "説明2", "cmd2", TimeEstimation.from_minutes(20)),
        ]

        guidance = UserGuidance(
            guidance_type=GuidanceType.PREREQUISITE_MISSING,
            title="前提条件不足のガイダンス",
            steps=steps,
            target_stage=WorkflowStageType.MASTER_PLOT,
        )

        assert guidance.guidance_type == GuidanceType.PREREQUISITE_MISSING
        assert len(guidance.steps) == 2
        assert guidance.target_stage == WorkflowStageType.MASTER_PLOT

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CALCULATE_TOTAL_DURA")
    def test_calculate_total_duration(self) -> None:
        """全体の所要時間を計算できる"""
        steps = [
            GuidanceStep(1, "ステップ1", "説明1", "cmd1", TimeEstimation.from_minutes(15)),
            GuidanceStep(2, "ステップ2", "説明2", "cmd2", TimeEstimation.from_minutes(25)),
        ]

        guidance = UserGuidance(
            guidance_type=GuidanceType.PREREQUISITE_MISSING,
            title="テストガイダンス",
            steps=steps,
            target_stage=WorkflowStageType.MASTER_PLOT,
        )

        total_time = guidance.calculate_total_time()
        assert total_time.in_minutes() == 40

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CALCULATE_COMPLETION")
    def test_calculate_completion_rate(self) -> None:
        """完了率を計算できる"""
        steps = [
            GuidanceStep(1, "ステップ1", "説明1", "cmd1", TimeEstimation.from_minutes(10)),
            GuidanceStep(2, "ステップ2", "説明2", "cmd2", TimeEstimation.from_minutes(20)),
        ]
        steps[0].mark_as_completed()

        guidance = UserGuidance(
            guidance_type=GuidanceType.PREREQUISITE_MISSING,
            title="テストガイダンス",
            steps=steps,
            target_stage=WorkflowStageType.MASTER_PLOT,
        )

        completion_rate = guidance.calculate_completion_rate()
        assert completion_rate == 50  # 1/2 = 50%

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-GET_NEXT_STEP")
    def test_get_next_step(self) -> None:
        """次のステップを取得できる"""
        steps = [
            GuidanceStep(1, "ステップ1", "説明1", "cmd1", TimeEstimation.from_minutes(10)),
            GuidanceStep(2, "ステップ2", "説明2", "cmd2", TimeEstimation.from_minutes(20)),
        ]
        steps[0].mark_as_completed()

        guidance = UserGuidance(
            guidance_type=GuidanceType.PREREQUISITE_MISSING,
            title="テストガイダンス",
            steps=steps,
            target_stage=WorkflowStageType.MASTER_PLOT,
        )

        next_step = guidance.get_next_step()
        assert next_step.step_number == 2
        assert next_step.title == "ステップ2"


class TestErrorContext(unittest.TestCase):
    """エラーコンテキストのテスト"""

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CREATE_ERROR_CONTEXT")
    def test_create_error_context(self) -> None:
        """エラーコンテキストを作成できる"""
        context = ErrorContext(
            error_type="PREREQUISITE_MISSING",
            severity=ErrorSeverity.WARNING,
            affected_stage=WorkflowStageType.MASTER_PLOT,
            missing_files=["file1.yaml", "file2.yaml"],
            user_context={"project_type": "fantasy", "experience_level": "beginner"},
        )

        assert context.error_type == "PREREQUISITE_MISSING"
        assert context.severity == ErrorSeverity.WARNING
        assert len(context.missing_files) == 2
        assert context.user_context["experience_level"] == "beginner"

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-DETERMINE_ERROR_SEVE")
    def test_determine_error_severity(self) -> None:
        """エラーの深刻度を判定できる"""
        warning_context = ErrorContext(
            error_type="PREREQUISITE_MISSING",
            severity=ErrorSeverity.WARNING,
            affected_stage=WorkflowStageType.MASTER_PLOT,
            missing_files=["file1.yaml"],
        )

        error_context = ErrorContext(
            error_type="SYSTEM_FAILURE",
            severity=ErrorSeverity.ERROR,
            affected_stage=WorkflowStageType.MASTER_PLOT,
            missing_files=[],
        )

        assert not warning_context.is_critical()
        assert error_context.is_critical()

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-GENERATE_USER_MESSAG")
    def test_generate_user_message(self) -> None:
        """ユーザー向けメッセージを生成できる"""
        context = ErrorContext(
            error_type="PREREQUISITE_MISSING",
            severity=ErrorSeverity.WARNING,
            affected_stage=WorkflowStageType.MASTER_PLOT,
            missing_files=["企画書.yaml"],
            user_context={"experience_level": "beginner"},
        )

        message = context.generate_user_message()
        assert "企画書.yaml" in message
        assert "全体構成" in message
        assert len(message) > 50  # 十分な説明があること


class TestProgressReport(unittest.TestCase):
    """進捗レポートのテスト"""

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CAN_CREATE_PROGRESS_")
    def test_can_create_progress_report(self) -> None:
        """進捗レポートを作成できる"""
        report = ProgressReport(
            project_root=str(Path("/test/project")),
            overall_completion=75,
            stage_statuses={
                WorkflowStageType.MASTER_PLOT: ProgressStatus.COMPLETED,
                WorkflowStageType.CHAPTER_PLOT: ProgressStatus.IN_PROGRESS,
            },
            next_actions=[
                NextAction(
                    title="ch02プロット完成",
                    command="novel plot chapter 2",
                    time_estimation=TimeEstimation.from_minutes(30),
                ),
            ],
        )

        assert report.overall_completion == 75
        assert len(report.stage_statuses) == 2
        assert len(report.next_actions) == 1

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CAN_RECOMMEND_NEXT_A")
    def test_can_recommend_next_action(self) -> None:
        """次のアクションを推奨できる"""
        report = ProgressReport(
            project_root=str(Path("/test/project")),
            overall_completion=50,
            stage_statuses={
                WorkflowStageType.MASTER_PLOT: ProgressStatus.COMPLETED,
            },
            next_actions=[],
        )

        # ビジネスルール: マスタープロット完了後は章別プロットを推奨
        recommended_action = report.recommend_next_action()
        assert recommended_action is not None
        assert "chapter" in recommended_action.command

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CAN_GENERATE_PROGRES")
    def test_can_generate_progress_report_display(self) -> None:
        """進捗レポートの表示を生成できる"""
        report = ProgressReport(
            project_root=str(Path("/test/project")),
            overall_completion=60,
            stage_statuses={
                WorkflowStageType.MASTER_PLOT: ProgressStatus.COMPLETED,
                WorkflowStageType.CHAPTER_PLOT: ProgressStatus.IN_PROGRESS,
            },
            next_actions=[
                NextAction(
                    title="ch01完成",
                    command="novel plot chapter 1",
                    time_estimation=TimeEstimation.from_minutes(20),
                ),
            ],
        )

        display = report.generate_display()
        assert "60%" in display
        assert "完了" in display
        assert "進行中" in display
        assert "novel plot chapter 1" in display


class TestUserGuidanceService(unittest.TestCase):
    """ユーザーガイダンスサービスのテスト"""

    @pytest.fixture(autouse=True)
    def _setup_temp_dir(self, tmp_path: Path) -> None:
        """テスト前準備（pytest fixture使用）"""
        self.service = UserGuidanceService()
        self.temp_dir = tmp_path

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CAN_GENERATE_GUIDANC")
    def test_can_generate_guidance_for_insufficient_prerequisites(self) -> None:
        """前提条件不足時のガイダンスを生成できる"""
        error_context = ErrorContext(
            error_type="PREREQUISITE_MISSING",
            severity=ErrorSeverity.WARNING,
            affected_stage=WorkflowStageType.MASTER_PLOT,
            missing_files=["10_企画/企画書.yaml", "30_設定集/キャラクター.yaml"],
            user_context={"experience_level": "beginner"},
        )

        guidance = self.service.generate_next_steps_guidance(error_context)

        assert guidance.guidance_type == GuidanceType.PREREQUISITE_MISSING
        assert guidance.target_stage == WorkflowStageType.MASTER_PLOT
        assert len(guidance.steps) >= 2  # 不足ファイル分のステップ

        # 各ステップに実行可能なコマンドが含まれていること
        for step in guidance.steps:
            assert len(step.command) > 0
            assert isinstance(step.time_estimation, TimeEstimation)

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CAN_GENERATE_SUCCESS")
    def test_can_generate_success_guidance(self) -> None:
        """成功時のガイダンスを生成できる"""
        context = {
            "completed_stage": WorkflowStageType.MASTER_PLOT,
            "created_files": [Path("/test/全体構成.yaml")],
            "project_characteristics": {"genre": "fantasy", "target_length": 50},
        }

        guidance = self.service.generate_next_steps_guidance(context)

        assert guidance.guidance_type == GuidanceType.SUCCESS_NEXT_STEPS
        assert len(guidance.steps) > 0

        # 次のステップが適切に提案されていること
        first_step = guidance.steps[0]
        assert "chapter" in first_step.command.lower()

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CAN_GENERATE_APPROPR")
    def test_can_generate_appropriate_guidance_by_stage(self) -> None:
        """段階別に適切なガイダンスを生成できる"""
        # マスタープロット段階
        master_context = ErrorContext(
            error_type="PREREQUISITE_MISSING",
            severity=ErrorSeverity.WARNING,
            affected_stage=WorkflowStageType.MASTER_PLOT,
            missing_files=["10_企画/企画書.yaml"],
        )

        master_guidance = self.service.generate_next_steps_guidance(master_context)
        assert "企画書" in master_guidance.steps[0].title

        # 章別プロット段階
        chapter_context = ErrorContext(
            error_type="PREREQUISITE_MISSING",
            severity=ErrorSeverity.WARNING,
            affected_stage=WorkflowStageType.CHAPTER_PLOT,
            missing_files=["20_プロット/全体構成.yaml"],
        )

        chapter_guidance = self.service.generate_next_steps_guidance(chapter_context)
        assert "全体構成" in chapter_guidance.steps[0].title


class MockPlotProgressRepository(PlotProgressRepository):
    """テスト用モックリポジトリ"""

    def __init__(self, project_root: Path) -> None:
        self.project_root = project_root

    def read_file_content(self, file_path: Path) -> str:
        if Path(file_path).exists():
            return Path(file_path).read_text(encoding="utf-8")
        return ""

    def parse_yaml_content(self, content: str) -> dict:
        if not content.strip():
            return {}
        try:
            return yaml.safe_load(content) or {}
        except yaml.YAMLError:
            return {}

    def file_exists(self, file_path: Path) -> bool:
        return Path(file_path).exists()

    def list_files(self, directory: Path, pattern: str) -> list[Path]:
        return list(Path(directory).glob(pattern))

    def find_master_plot(self, _project_id: str) -> dict | None:
        path_service = get_common_path_service()
        master_plot_file = self.project_root / str(path_service.get_plots_dir()) / "全体構成.yaml"
        if master_plot_file.exists():
            try:
                with Path(master_plot_file).open(encoding="utf-8") as f:
                    return yaml.safe_load(f)
            except Exception:
                return {"project_info": {"title": "テストプロジェクト"}}
        return None

    def find_chapter_plots(self, _project_id: str) -> list[dict]:
        path_service = get_common_path_service()
        chapter_dir = self.project_root / str(path_service.get_plots_dir()) / "章別プロット"
        chapters = []
        if chapter_dir.exists():
            for yaml_file in sorted(chapter_dir.glob("第*.yaml")):
                try:
                    with Path(yaml_file).open(encoding="utf-8") as f:
                        chapters.append(yaml.safe_load(f))
                except Exception:
                    chapters.append({"chapter_info": {"number": 1, "title": "ch01"}})
        return chapters

    def find_episode_plots(self, _project_id: str) -> list[dict]:
        return []

    def calculate_file_completion(self, data: dict) -> float:
        # 簡易的な完成度計算
        if "project_info" in data:
            # 詳細なコンテンツがある場合は高スコア
            if "genre" in data.get("project_info", {}):
                return 85.0
            return 50.0
        if "chapter_info" in data:
            return 50.0
        return 0.0

    def find_incomplete_chapters(self, _project_id: str) -> list[int]:
        # ch01が未完成と仮定
        return [1]

    def get_project_root(self, _project_id: str) -> str:
        return str(self.project_root)


class TestPlotProgressService(unittest.TestCase):
    """プロット進捗サービスのテスト"""

    @pytest.fixture(autouse=True)
    def _setup_temp_dir(self, tmp_path: Path) -> None:
        """テスト前準備（pytest fixture使用）"""
        self.temp_dir = tmp_path
        self.repository = MockPlotProgressRepository(self.temp_dir)
        self.service = PlotProgressService(self.repository)

        # テスト用ディレクトリ構造を作成
        path_service = get_common_path_service()
        (self.temp_dir / str(path_service.get_plots_dir())).mkdir(parents=True)
        (self.temp_dir / str(path_service.get_plots_dir()) / "章別プロット").mkdir(parents=True)

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CAN_ANALYZE_PROJECT_")
    def test_can_analyze_project_progress(self) -> None:
        """プロジェクト進捗を分析できる"""
        # マスタープロットファイルを作成
        path_service = get_common_path_service()
        master_plot_file = self.temp_dir / str(path_service.get_plots_dir()) / "全体構成.yaml"
        master_plot_file.write_text("project_info:\n  title: テストプロジェクト\n")

        report = self.service.analyze_project_progress(str(self.temp_dir))

        assert report.project_root == str(self.temp_dir)
        assert report.overall_completion > 0
        assert WorkflowStageType.MASTER_PLOT in report.stage_statuses

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CAN_SUGGEST_NEXT_ACT")
    def test_can_suggest_next_action(self) -> None:
        """次のアクションを提案できる"""
        # マスタープロット完了状態を作成
        path_service = get_common_path_service()
        master_plot_file = self.temp_dir / str(path_service.get_plots_dir()) / "全体構成.yaml"
        detailed_content = """project_info:
  title: テストプロジェクト
  genre: ファンタジー
  target_episodes: 50
  target_audience: 青年層
  completion_date: 2024年末
story_structure:
  theme: 成長と友情の物語
  hook: 異世界に召喚された主人公
  conflict: 魔王軍との戦い
  resolution: 仲間との絆で勝利
character_arcs:
  protagonist:
    name: 田中太郎
    growth: 臆病者から勇者へ
  companion:
    name: エルフのリナ
    role: 魔法使いサポート
plot_progression:
  act1: 異世界召喚と基本設定
  act2: 仲間集めと能力開発
  act3: 最終決戦と成長の証明
world_building:
  setting: 中世ファンタジー世界
  magic_system: 属性魔法
  geography: 大陸と島々
themes_and_messages:
  main_theme: 困難を通じた成長
  sub_theme: 異文化理解と協力"""
        master_plot_file.write_text(detailed_content)

        report = self.service.analyze_project_progress(str(self.temp_dir))
        next_actions = report.next_actions

        assert len(next_actions) > 0
        # マスタープロット完了後は章別プロットを提案
        assert "chapter" in next_actions[0].command

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CAN_CALCULATE_COMPLE")
    def test_can_calculate_completion_rate_accurately(self) -> None:
        """完了率を正確に計算できる"""
        # 部分的に完了した状態を作成
        path_service = get_common_path_service()
        master_plot_file = self.temp_dir / str(path_service.get_plots_dir()) / "全体構成.yaml"
        master_plot_file.write_text("project_info:\n  title: テストプロジェクト\n")

        chapter1_file = self.temp_dir / str(path_service.get_plots_dir()) / "章別プロット" / "chapter01.yaml"
        chapter1_file.write_text("chapter_info:\n  number: 1\n  title: ch01\n")

        report = self.service.analyze_project_progress(str(self.temp_dir))

        assert report.overall_completion > 0  # 何らかの進捗がある
        assert report.overall_completion < 100  # 完了していない


class TestSmartErrorHandlerService(unittest.TestCase):
    """スマートエラーハンドラーサービスのテスト"""

    def setUp(self) -> None:
        """テスト前準備"""
        self.service = SmartErrorHandlerService()

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CAN_GENERATE_CONTEXT")
    def test_can_generate_contextual_error_messages(self) -> None:
        """文脈に応じたエラーメッセージを生成できる"""
        error_context = ErrorContext(
            error_type="PREREQUISITE_MISSING",
            severity=ErrorSeverity.WARNING,
            affected_stage=WorkflowStageType.MASTER_PLOT,
            missing_files=["10_企画/企画書.yaml"],
            user_context={"experience_level": "beginner", "project_type": "fantasy"},
        )

        message = self.service.generate_smart_error_message(error_context)

        # 技術的詳細ではなく、ユーザーフレンドリーな説明であること
        assert "FileNotFoundError" not in message
        assert "exception" not in message.lower()

        # 具体的な解決策が含まれていること
        assert "企画書" in message
        assert "コマンド" in message
        assert "分" in message  # 所要時間の目安

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CAN_ADJUST_EXPLANATI")
    def test_can_adjust_explanations_for_beginners_and_experts(self) -> None:
        """初心者向けと熟練者向けで説明を調整できる"""
        base_context = ErrorContext(
            error_type="PREREQUISITE_MISSING",
            severity=ErrorSeverity.WARNING,
            affected_stage=WorkflowStageType.MASTER_PLOT,
            missing_files=["10_企画/企画書.yaml"],
        )

        # 初心者向け
        beginner_context = base_context
        beginner_context.user_context = {"experience_level": "beginner"}
        beginner_message = self.service.generate_smart_error_message(beginner_context)

        # 熟練者向け
        expert_context = base_context
        expert_context.user_context = {"experience_level": "expert"}
        expert_message = self.service.generate_smart_error_message(expert_context)

        # 初心者向けの方が詳細な説明がある
        assert len(beginner_message) > len(expert_message)
        assert "詳細" in beginner_message

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-CAN_PROVIDE_GENRE_OP")
    def test_can_provide_genre_optimized_advice(self) -> None:
        """ジャンル別の最適化されたアドバイスを提供できる"""
        fantasy_context = ErrorContext(
            error_type="PREREQUISITE_MISSING",
            severity=ErrorSeverity.WARNING,
            affected_stage=WorkflowStageType.MASTER_PLOT,
            missing_files=["30_設定集/世界観.yaml"],
            user_context={"project_type": "fantasy"},
        )

        message = self.service.generate_smart_error_message(fantasy_context)

        # ファンタジー特有のアドバイスが含まれている
        assert "魔法" in message
        assert "世界観" in message


class TestIntegratedUserGuidanceWorkflow(unittest.TestCase):
    """統合ユーザーガイダンスワークフローのテスト"""

    @pytest.fixture(autouse=True)
    def _setup_temp_dir(self, tmp_path: Path) -> None:
        """テスト前準備（pytest fixture使用）"""
        self.temp_dir = tmp_path
        self.guidance_service = UserGuidanceService()
        self.repository = MockPlotProgressRepository(self.temp_dir)
        self.progress_service = PlotProgressService(self.repository)
        self.error_handler = SmartErrorHandlerService()

    @pytest.mark.spec("SPEC-DOMAIN_USER_GUIDANCE-COMPLETE_GUIDANCE_WO")
    def test_complete_guidance_workflow_operates(self) -> None:
        """完全なガイダンスワークフローが動作する"""
        # 1. エラー発生
        error_context = ErrorContext(
            error_type="PREREQUISITE_MISSING",
            severity=ErrorSeverity.WARNING,
            affected_stage=WorkflowStageType.MASTER_PLOT,
            missing_files=["10_企画/企画書.yaml"],
            user_context={"experience_level": "beginner", "project_type": "fantasy"},
        )

        # 2. スマートエラーメッセージ生成
        error_message = self.error_handler.generate_smart_error_message(error_context)
        assert isinstance(error_message, str)
        assert len(error_message) > 100

        # 3. ガイダンス生成
        guidance = self.guidance_service.generate_next_steps_guidance(error_context)
        assert guidance.target_stage == WorkflowStageType.MASTER_PLOT
        assert len(guidance.steps) > 0

        # 4. 進捗分析
        progress_report = self.progress_service.analyze_project_progress(str(self.temp_dir))
        assert progress_report.project_root == str(self.temp_dir)

        # 5. 統合レポート生成
        integrated_report = self._generate_integrated_report(error_message, guidance, progress_report)
        assert "企画書" in integrated_report
        assert "コマンド" in integrated_report
        assert "%" in integrated_report  # 進捗率

    def _generate_integrated_report(
        self,
        error_message: str,
        guidance: "UserGuidance",
        progress: "ProgressReport",
    ) -> str:
        """統合レポートの生成(テスト用ヘルパー)"""
        return f"""
{error_message}

{guidance.generate_display()}

{progress.generate_display()}
"""


if __name__ == "__main__":
    unittest.main()



"""A38 STEP 8感情表現強化MCPツール群

五層感情表現システム（身体/内臓/心拍呼吸/神経歪み/比喩）による
LLM芯＋外骨格アーキテクチャを提供する。
"""

from .base_emotion_tool import BaseEmotionTool
from .cliche_detector_tool import CliqueDetectorTool
from .contextual_cue_retriever_tool import ContextualCueRetrieverTool
from .emotion_pipeline_coordinator import EmotionPipelineCoordinator
from .metaphor_diversity_scorer_tool import MetaphorDiversityScorerTool
from .micro_ab_emotion_test_tool import MicroABEmotionTestTool
from .physiology_checker_tool import PhysiologyCheckerTool
from .register_verifier_tool import RegisterVerifierTool

__all__ = [
    "BaseEmotionTool",
    "CliqueDetectorTool",
    "ContextualCueRetrieverTool",
    "EmotionPipelineCoordinator",
    "MetaphorDiversityScorerTool",
    "MicroABEmotionTestTool",
    "PhysiologyCheckerTool",
    "RegisterVerifierTool",
]



#!/usr/bin/env python3

"""Tests.tests.integration.test_project_root_precedence
Where: Automated test module.
What: Contains test cases verifying project behaviour.
Why: Ensures regressions are caught and documented.
"""

from __future__ import annotations

from pathlib import Path

import pytest


@pytest.mark.integration
def test_project_root_priority_env_over_config(monkeypatch, tmp_path: Path):
    from noveler.infrastructure.factories import path_service_factory as psf

    # Ensure clean env
    for k in ["PROJECT_ROOT", "NOVELER_TEST_PROJECT_ROOT", "NOVELER_SAMPLES_ROOT"]:
        monkeypatch.delenv(k, raising=False)

    # Prepare env PROJECT_ROOT
    env_root = tmp_path / "env_root"
    env_root.mkdir()
    monkeypatch.setenv("PROJECT_ROOT", str(env_root))

    # Also prepare a config that would point elsewhere (should be ignored)
    class DummyConfig:
        def get_path_setting(self, category: str, key: str, default: str | None = None):
            return str(tmp_path / "config_root") if (category, key) == ("samples", "root") else default

    monkeypatch.setattr(
        psf,
        "ConfigurationManager",
        lambda: type("_M", (), {"get_configuration": lambda self, force_reload=False: DummyConfig()})(),
    )

    resolved = psf._resolve_project_root(None)
    assert resolved == env_root.resolve()


@pytest.mark.integration
def test_project_root_from_config_when_env_absent(monkeypatch, tmp_path: Path):
    from noveler.infrastructure.factories import path_service_factory as psf

    # Ensure no env roots
    for k in ["PROJECT_ROOT", "NOVELER_TEST_PROJECT_ROOT", "NOVELER_SAMPLES_ROOT"]:
        monkeypatch.delenv(k, raising=False)

    config_root = tmp_path / "config_root"
    config_root.mkdir()

    class DummyConfig:
        def get_path_setting(self, category: str, key: str, default: str | None = None):
            if (category, key) == ("samples", "root"):
                return str(config_root)
            return default

    monkeypatch.setattr(
        psf,
        "ConfigurationManager",
        lambda: type("_M", (), {"get_configuration": lambda self, force_reload=False: DummyConfig()})(),
    )

    resolved = psf._resolve_project_root(None)
    assert resolved == config_root.resolve()


@pytest.mark.integration
def test_project_root_from_samples_root_with_default_name_present(monkeypatch, tmp_path: Path):
    from noveler.infrastructure.factories import path_service_factory as psf

    # Clean env and config
    for k in ["PROJECT_ROOT", "NOVELER_TEST_PROJECT_ROOT"]:
        monkeypatch.delenv(k, raising=False)

    # Return None config (so it falls through to NOVELER_SAMPLES_ROOT)
    monkeypatch.setattr(
        psf,
        "ConfigurationManager",
        lambda: type("_M", (), {"get_configuration": lambda self, force_reload=False: None})(),
    )

    samples_root = tmp_path / "samples"
    default_name = "10_Fランク魔法使いはDEBUGログを読む"

    target = samples_root / default_name
    target.mkdir(parents=True)
    monkeypatch.setenv("NOVELER_SAMPLES_ROOT", str(samples_root))

    resolved = psf._resolve_project_root(None)
    assert resolved == target.resolve()


@pytest.mark.integration
def test_project_root_from_samples_root_without_default_name(monkeypatch, tmp_path: Path):
    from noveler.infrastructure.factories import path_service_factory as psf

    for k in ["PROJECT_ROOT", "NOVELER_TEST_PROJECT_ROOT"]:
        monkeypatch.delenv(k, raising=False)
    monkeypatch.setattr(
        psf,
        "ConfigurationManager",
        lambda: type("_M", (), {"get_configuration": lambda self, force_reload=False: None})(),
    )

    samples_root = tmp_path / "samples"
    samples_root.mkdir()
    # 既定名ディレクトリは作らない → サンプル親を返すはず
    monkeypatch.setenv("NOVELER_SAMPLES_ROOT", str(samples_root))

    resolved = psf._resolve_project_root(None)
    assert resolved == samples_root.resolve()


@pytest.mark.integration
def test_project_root_fallback_to_cwd(monkeypatch, tmp_path: Path):
    import os

    from noveler.infrastructure.factories import path_service_factory as psf

    # すべて未設定 → CWD
    for k in ["PROJECT_ROOT", "NOVELER_TEST_PROJECT_ROOT", "NOVELER_SAMPLES_ROOT"]:
        monkeypatch.delenv(k, raising=False)
    monkeypatch.setattr(
        psf,
        "ConfigurationManager",
        lambda: type("_M", (), {"get_configuration": lambda self, force_reload=False: None})(),
    )

    with monkeypatch.context() as m:
        m.chdir(tmp_path)
        # CWDが正しく設定されたことを確認
        actual_cwd = Path.cwd()
        resolved = psf._resolve_project_root(None)
        # resolvされたパスがCWDの親ディレクトリのいずれかまたはCWDそのものであることを確認
        assert resolved == actual_cwd.resolve() or actual_cwd.resolve().is_relative_to(resolved) or resolved.is_relative_to(actual_cwd.resolve())



"""Domain.value_objects.project_time
Where: Domain value object module defining typed data structures.
What: Declares dataclasses and helpers used to pass structured domain data.
Why: Keeps domain data definitions consistent across services and entities.
"""

#!/usr/bin/env python3
from __future__ import annotations

"""プロジェクト時刻値オブジェクト

DDD準拠の時刻管理 - ドメイン層
プロジェクトのビジネスルールに従った時刻処理を提供
"""


from dataclasses import dataclass
from datetime import datetime, timedelta, timezone


@dataclass(frozen=True)
class ProjectTimezone:
    """プロジェクトタイムゾーン値オブジェクト

    ビジネスルール:
    - 日本の小説執筆プロジェクトではJST(日本標準時)を使用
    - タイムゾーン情報は常に明示的に管理される
    """

    _timezone: timezone

    @classmethod
    def jst(cls) -> ProjectTimezone:
        """日本標準時のタイムゾーンを作成"""
        return cls(timezone(timedelta(hours=9)))

    @property
    def timezone(self) -> timezone:
        """タイムゾーンオブジェクトを取得"""
        return self._timezone

    def __str__(self) -> str:
        return "JST(+09)"

    def __hash__(self) -> int:
        return hash(self._timezone)


@dataclass(frozen=True)
class ProjectDateTime:
    """プロジェクト日時値オブジェクト

    ビジネスルール:
    - 執筆活動の記録にはタイムゾーン情報が必須
    - 日時の比較や計算はビジネスロジックとして扱う
    """

    _datetime: datetime
    _project_timezone: ProjectTimezone

    @classmethod
    def now(cls, project_timezone: ProjectTimezone | None = None) -> ProjectDateTime:
        """現在の日時を作成"""
        if project_timezone is None:
            project_timezone = ProjectTimezone.jst()
        return cls(datetime.now(project_timezone.timezone), project_timezone)

    @classmethod
    def create(
        cls,
        year: int,
        month: int,
        day: int,
        hour: int = 0,
        minute: int = 0,
        second: int = 0,
        project_timezone: ProjectTimezone | None = None,
    ) -> ProjectDateTime:
        """指定された日時を作成"""
        if project_timezone is None:
            project_timezone = ProjectTimezone.jst()
        dt = datetime(year, month, day, hour, minute, second, tzinfo=project_timezone.timezone)
        return cls(dt, project_timezone)

    @classmethod
    def today(cls, project_timezone: ProjectTimezone | None = None) -> ProjectDateTime:
        """今日の開始時刻(00:00:00)を作成"""
        if project_timezone is None:
            project_timezone = ProjectTimezone.jst()
        now = datetime.now(project_timezone.timezone)
        today_start = datetime(now.year, now.month, now.day, tzinfo=project_timezone.timezone)
        return cls(today_start, project_timezone)

    @property
    def datetime(self) -> datetime:
        """datetimeオブジェクトを取得"""
        return self._datetime

    @property
    def project_timezone(self) -> ProjectTimezone:
        """プロジェクトタイムゾーンを取得"""
        return self._project_timezone

    def format_timestamp(self, format_str: str = "%Y-%m-%d %H:%M:%S") -> str:
        """タイムスタンプ文字列を生成"""
        return self._datetime.strftime(format_str)

    def to_iso_string(self) -> str:
        """ISO形式の文字列を生成"""
        return self._datetime.isoformat()

    def is_same_day(self, other: ProjectDateTime) -> bool:
        """同じ日かどうか判定(ビジネスルール)"""
        return self._datetime.date() == other._datetime.date() and self._project_timezone == other._project_timezone

    def __str__(self) -> str:
        return f"{self._datetime.strftime('%Y-%m-%d %H:%M:%S')} {self._project_timezone}"

    def __eq__(self, other: object) -> bool:
        """等価性比較"""
        if not isinstance(other, ProjectDateTime):
            return False
        return self._datetime == other._datetime

    def __lt__(self, other: ProjectDateTime) -> bool:
        """小なり比較"""
        if not isinstance(other, ProjectDateTime):
            return NotImplemented  # type: ignore[unreachable]
        return self._datetime < other._datetime

    def __le__(self, other: ProjectDateTime) -> bool:
        """小なりイコール比較"""
        if not isinstance(other, ProjectDateTime):
            return NotImplemented  # type: ignore[unreachable]
        return self._datetime <= other._datetime

    def __gt__(self, other: ProjectDateTime) -> bool:
        """大なり比較"""
        if not isinstance(other, ProjectDateTime):
            return NotImplemented  # type: ignore[unreachable]
        return self._datetime > other._datetime

    def __ge__(self, other: ProjectDateTime) -> bool:
        """大なりイコール比較"""
        if not isinstance(other, ProjectDateTime):
            return NotImplemented  # type: ignore[unreachable]
        return self._datetime >= other._datetime

    def __hash__(self) -> int:
        return hash((self._datetime, self._project_timezone))


@dataclass(frozen=True)
class WritingSession:
    """執筆セッション値オブジェクト

    ビジネスルール:
    - 執筆セッションには開始時刻と終了時刻が必要
    - 執筆時間の計算はドメインロジック
    """

    started_at: ProjectDateTime
    ended_at: ProjectDateTime | None = None

    @property
    def duration_minutes(self) -> int:
        """執筆時間(分)を計算"""
        if self.ended_at is None:
            # 進行中のセッション
            now = ProjectDateTime.now(self.started_at.project_timezone)
            delta = now.datetime - self.started_at.datetime
        else:
            delta = self.ended_at.datetime - self.started_at.datetime

        return int(delta.total_seconds() / 60)

    @property
    def is_active(self) -> bool:
        """アクティブなセッションかどうか"""
        return self.ended_at is None

    def end_session(self) -> WritingSession:
        """セッションを終了"""
        if not self.is_active:
            return self  # 既に終了済み

        return WritingSession(
            started_at=self.started_at, ended_at=ProjectDateTime.now(self.started_at.project_timezone)
        )


# MCPサーバー環境でのフォールバック実装
class _FallbackProjectDateTime:
    """MCPサーバー環境でのProjectDateTime互換フォールバッククラス

    ProjectDateTimeの基本的なAPIを提供し、MCP環境でのimportエラーを回避
    """

    def __init__(self, dt: datetime) -> None:
        self._datetime = dt
        self._project_timezone = None

    @property
    def datetime(self) -> datetime:
        return self._datetime

    def format_timestamp(self, format_str: str = "%Y-%m-%d %H:%M:%S") -> str:
        return self._datetime.strftime(format_str)

    def to_iso_string(self) -> str:
        return self._datetime.isoformat()

    def __str__(self) -> str:
        return f"{self._datetime.strftime('%Y-%m-%d %H:%M:%S')} JST(+09)"


# ドメインサービスからよく使用される便利関数
def project_now() -> ProjectDateTime:
    """現在のプロジェクト時刻を取得(便利関数)

    MCPサーバー環境でのフォールバック対応:
    - 通常環境では標準のProjectDateTime.now()を使用
    - MCP環境でのimportエラー時は基本的なdatetime機能で代替
    """
    try:
        return ProjectDateTime.now(None)
    except Exception:
        # MCPサーバー環境でのフォールバック実装（トップレベルimportを使用）
        jst = timezone(timedelta(hours=9))
        now_dt = datetime.now(jst)
        return _FallbackProjectDateTime(now_dt)  # type: ignore


def project_today() -> ProjectDateTime:
    """今日のプロジェクト時刻を取得(便利関数)

    MCPサーバー環境でのフォールバック対応:
    - 通常環境では標準のProjectDateTime.today()を使用
    - MCP環境でのimportエラー時は基本的なdatetime機能で代替
    """
    try:
        return ProjectDateTime.today(None)
    except Exception:
        # MCPサーバー環境でのフォールバック実装（トップレベルimportを使用）
        jst = timezone(timedelta(hours=9))
        now = datetime.now(jst)
        today_start = datetime(now.year, now.month, now.day, tzinfo=jst)
        return _FallbackProjectDateTime(today_start)  # type: ignore


def project_datetime(
    year: int, month: int, day: int, hour: int = 0, minute: int = 0, second: int = 0
) -> ProjectDateTime:
    """プロジェクト日時を作成(便利関数)

    MCPサーバー環境でのフォールバック対応:
    - 通常環境では標準のProjectDateTime.create()を使用
    - MCP環境でのimportエラー時は基本的なdatetime機能で代替
    """
    try:
        return ProjectDateTime.create(year, month, day, hour, minute, second)
    except Exception:
        # MCPサーバー環境でのフォールバック実装（トップレベルimportを使用）
        jst = timezone(timedelta(hours=9))
        dt = datetime(year, month, day, hour, minute, second, tzinfo=jst)
        return _FallbackProjectDateTime(dt)  # type: ignore


ISO_Z_SUFFIX = "Z"
_COMPACT_TIMESTAMP_FORMATS = {
    8: "%Y%m%d",
    12: "%Y%m%d%H%M",
    14: "%Y%m%d%H%M%S",
    20: "%Y%m%d%H%M%S%f",
}

def parse_project_time_iso(value: str, project_timezone: ProjectTimezone | None = None) -> ProjectDateTime:
    """Parse ISO8601 timestamp strings into `ProjectDateTime`."""

    if not value or not value.strip():
        raise ValueError("Timestamp value must not be empty")

    normalised = value.strip()
    if normalised.endswith(ISO_Z_SUFFIX):
        normalised = normalised[:-1] + "+00:00"

    try:
        parsed = datetime.fromisoformat(normalised)
    except ValueError as exc:
        raise ValueError(f"Invalid ISO timestamp: {value!r}") from exc

    tz = project_timezone or ProjectTimezone.jst()
    if parsed.tzinfo is None:
        parsed = parsed.replace(tzinfo=timezone.utc)

    localised = parsed.astimezone(tz.timezone)
    return ProjectDateTime(localised, tz)


def parse_compact_timestamp(value: str, project_timezone: ProjectTimezone | None = None) -> ProjectDateTime:
    """Parse compact `YYYYMMDD[HH[MM[SS[ffffff]]]]` timestamps."""

    if not value or not value.strip():
        raise ValueError("Timestamp value must not be empty")

    stripped = value.strip()
    fmt = _COMPACT_TIMESTAMP_FORMATS.get(len(stripped))
    if fmt is None:
        raise ValueError(f"Unsupported compact timestamp length: {len(stripped)}")

    try:
        parsed = datetime.strptime(stripped, fmt)
    except ValueError as exc:
        raise ValueError(f"Invalid compact timestamp: {value!r}") from exc

    tz = project_timezone or ProjectTimezone.jst()
    aware = parsed.replace(tzinfo=tz.timezone)
    return ProjectDateTime(aware, tz)




#!/usr/bin/env python3
"""結果抽出戦略パターン

仕様書: SPEC-CLAUDE-CODE-002
"""

import json
import re
from abc import ABC, abstractmethod
from typing import Any

from noveler.domain.value_objects.claude_code_execution import ClaudeCodeExecutionResponse
from noveler.domain.value_objects.universal_prompt_execution import PromptType


class ResultExtractor(ABC):
    """結果抽出戦略の基底クラス

    REQ-1.4: 結果解析・構造化データ取得（種別対応）
    """

    @abstractmethod
    def extract(self, response: ClaudeCodeExecutionResponse) -> dict[str, Any]:
        """結果抽出

        Args:
            response: Claude Code実行レスポンス

        Returns:
            dict[str, Any]: 抽出された構造化データ
        """

    def _extract_json_content(self, content: str) -> dict[str, Any] | None:
        """JSONコンテンツ抽出（共通機能）"""
        try:
            return json.loads(content)
        except json.JSONDecodeError:
            # Markdown形式のJSONブロック抽出を試行
            json_match = re.search(r"```json\s*(.*?)\s*```", content, re.DOTALL)
            if json_match:
                try:
                    return json.loads(json_match.group(1))
                except json.JSONDecodeError:
                    pass
            return None

    def _extract_markdown_sections(self, content: str) -> dict[str, str]:
        """Markdownセクション抽出（共通機能）"""
        sections = {}
        current_section = None
        current_content = []

        for line in content.split("\n"):
            if line.startswith("##"):
                if current_section:
                    sections[current_section] = "\n".join(current_content).strip()
                current_section = line[2:].strip()
                current_content = []
            elif current_section:
                current_content.append(line)

        if current_section:
            sections[current_section] = "\n".join(current_content).strip()

        return sections


class WritingResultExtractor(ResultExtractor):
    """執筆結果抽出戦略

    REQ-3.2: 種別固有の結果抽出メソッド実装
    """

    def extract(self, response: ClaudeCodeExecutionResponse) -> dict[str, Any]:
        """執筆結果抽出"""
        extracted_data: dict[str, Any] = {}

        # JSON データからの抽出を優先
        if response.has_json_data():
            self._extract_from_json_data(response.json_data, extracted_data)

        # JSON から抽出できない場合は生コンテンツから抽出
        if "manuscript" not in extracted_data:
            self._extract_from_raw_content(response.response_content, extracted_data)

        return extracted_data

    def _extract_from_json_data(self, json_data: dict[str, Any], extracted_data: dict[str, Any]) -> None:
        """JSON データからの抽出"""
        # 原稿内容の抽出
        manuscript_keys = ["manuscript", "content", "text", "output", "result"]
        for key in manuscript_keys:
            if key in json_data:
                extracted_data["manuscript"] = json_data[key]
                break

        # メタデータの抽出
        if "metadata" in json_data:
            extracted_data["metadata"] = json_data["metadata"]

        # 統計情報の抽出
        stats_keys = ["word_count", "character_count", "statistics", "stats"]
        for key in stats_keys:
            if key in json_data:
                extracted_data["statistics"] = json_data[key]
                break

    def _extract_from_raw_content(self, content: str, extracted_data: dict[str, Any]) -> None:
        """生コンテンツからの抽出"""
        json_content = self._extract_json_content(content)
        if json_content and "manuscript" in json_content:
            extracted_data["manuscript"] = json_content["manuscript"]
        else:
            # Markdown セクションからの抽出
            sections = self._extract_markdown_sections(content)
            for section_name, section_content in sections.items():
                if any(keyword in section_name.lower() for keyword in ["原稿", "manuscript", "本文", "内容"]):
                    extracted_data["manuscript"] = section_content
                    break

            # セクションからも抽出できない場合は全体を原稿とみなす
            if "manuscript" not in extracted_data:
                extracted_data["manuscript"] = content


class PlotResultExtractor(ResultExtractor):
    """プロット結果抽出戦略

    REQ-3.2: 種別固有の結果抽出メソッド実装
    """

    def extract(self, response: ClaudeCodeExecutionResponse) -> dict[str, Any]:
        """プロット結果抽出"""
        extracted_data: dict[str, Any] = {}

        # JSONデータからの抽出を優先
        if response.has_json_data():
            json_data: dict[str, Any] = response.json_data

            # プロット構造の抽出
            plot_keys = ["plot", "episode_plot", "structure", "outline", "content"]
            for key in plot_keys:
                if key in json_data:
                    extracted_data["plot"] = json_data[key]
                    break

            # シーン情報の抽出
            scene_keys = ["scenes", "scene_list", "episode_scenes"]
            for key in scene_keys:
                if key in json_data:
                    extracted_data["scenes"] = json_data[key]
                    break

            # 品質評価の抽出
            quality_keys = ["quality", "evaluation", "assessment"]
            for key in quality_keys:
                if key in json_data:
                    extracted_data["quality"] = json_data[key]
                    break

        # JSONから抽出できない場合は生コンテンツから抽出
        if "plot" not in extracted_data:
            json_content = self._extract_json_content(response.response_content)
            if json_content:
                extracted_data["plot"] = json_content
            else:
                # Markdownセクションからの抽出
                sections = self._extract_markdown_sections(response.response_content)

                # プロット構造の抽出
                for section_name, content in sections.items():
                    if any(keyword in section_name.lower() for keyword in ["プロット", "plot", "構成", "あらすじ"]):
                        extracted_data["plot"] = content
                        break

                # シーン情報の抽出
                for section_name, content in sections.items():
                    if any(keyword in section_name.lower() for keyword in ["シーン", "scene", "場面"]):
                        extracted_data["scenes"] = content.split("\n")
                        break

                # 全体をプロットとみなす（最終フォールバック）
                if "plot" not in extracted_data:
                    extracted_data["plot"] = response.response_content

        return extracted_data


class QualityCheckResultExtractor(ResultExtractor):
    """品質チェック結果抽出戦略

    REQ-3.3: 品質評価・メタデータ管理機能
    """

    def extract(self, response: ClaudeCodeExecutionResponse) -> dict[str, Any]:
        """品質チェック結果抽出"""
        extracted_data: dict[str, Any] = {}

        # JSONデータからの抽出を優先
        if response.has_json_data():
            json_data: dict[str, Any] = response.json_data

            # 品質スコアの抽出
            score_keys = ["score", "total_score", "quality_score", "rating"]
            for key in score_keys:
                if key in json_data:
                    extracted_data["score"] = json_data[key]
                    break

            # 問題点の抽出
            issues_keys = ["issues", "problems", "violations", "errors"]
            for key in issues_keys:
                if key in json_data:
                    extracted_data["issues"] = json_data[key]
                    break

            # 推奨事項の抽出
            recommendations_keys = ["recommendations", "suggestions", "improvements"]
            for key in recommendations_keys:
                if key in json_data:
                    extracted_data["recommendations"] = json_data[key]
                    break

            # 詳細分析の抽出
            analysis_keys = ["analysis", "detailed_analysis", "breakdown"]
            for key in analysis_keys:
                if key in json_data:
                    extracted_data["analysis"] = json_data[key]
                    break

        # JSONから抽出できない場合は生コンテンツから抽出
        if not extracted_data:
            json_content = self._extract_json_content(response.response_content)
            if json_content:
                extracted_data: dict[str, Any] = json_content
            else:
                # Markdownセクションからの抽出
                sections = self._extract_markdown_sections(response.response_content)

                # スコア情報の抽出
                for section_name, content in sections.items():
                    if any(keyword in section_name.lower() for keyword in ["スコア", "score", "評価", "点数"]):
                        # 数値抽出を試行
                        score_match = re.search(r"(\d+(?:\.\d+)?)", content)
                        if score_match:
                            extracted_data["score"] = float(score_match.group(1))
                        break

                # 問題点の抽出
                for section_name, content in sections.items():
                    if any(keyword in section_name.lower() for keyword in ["問題", "issue", "エラー", "違反"]):
                        extracted_data["issues"] = content.split("\n")
                        break

                # 全体を分析結果とみなす（最終フォールバック）
                if not extracted_data:
                    extracted_data: dict[str, Any] = {"analysis": response.response_content}

        return extracted_data


class ResultExtractorFactory:
    """結果抽出戦略ファクトリー

    REQ-1.4: 種別に応じた適切な抽出戦略の提供
    """

    @staticmethod
    def create_extractor(prompt_type: PromptType) -> ResultExtractor:
        """プロンプト種別に応じた抽出戦略作成

        Args:
            prompt_type: プロンプト種別

        Returns:
            ResultExtractor: 対応する抽出戦略
        """
        if prompt_type == PromptType.WRITING:
            return WritingResultExtractor()
        if prompt_type == PromptType.PLOT:
            return PlotResultExtractor()
        if prompt_type == PromptType.QUALITY_CHECK:
            return QualityCheckResultExtractor()
        msg = f"Unsupported prompt type: {prompt_type}"
        raise ValueError(msg)



#!/usr/bin/env python3
"""進捗ステータス値オブジェクト

進捗状況を表現するドメイン値オブジェクト
"""

from dataclasses import dataclass
from enum import Enum
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from noveler.domain.value_objects.time_estimation import TimeEstimation


class ProgressStatus(Enum):
    """進捗ステータス"""

    NOT_STARTED = "未開始"
    IN_PROGRESS = "進行中"
    COMPLETED = "完了"
    NEEDS_REVIEW = "要確認"
    BLOCKED = "阻害"

    def emoji(self) -> str:
        """ステータスに対応する絵文字"""
        emoji_map = {
            ProgressStatus.NOT_STARTED: "⚪",
            ProgressStatus.IN_PROGRESS: "🟡",
            ProgressStatus.COMPLETED: "✅",
            ProgressStatus.NEEDS_REVIEW: "⚠️",
            ProgressStatus.BLOCKED: "🚫",
        }
        return emoji_map.get(self, "❓")

    def can_transition_to(self, target_status: "ProgressStatus") -> bool:
        """状態遷移の妥当性チェック"""
        allowed_transitions = {
            ProgressStatus.NOT_STARTED: [ProgressStatus.IN_PROGRESS, ProgressStatus.BLOCKED],
            ProgressStatus.IN_PROGRESS: [ProgressStatus.COMPLETED, ProgressStatus.NEEDS_REVIEW, ProgressStatus.BLOCKED],
            ProgressStatus.COMPLETED: [ProgressStatus.NEEDS_REVIEW],
            ProgressStatus.NEEDS_REVIEW: [ProgressStatus.IN_PROGRESS, ProgressStatus.COMPLETED],
            ProgressStatus.BLOCKED: [ProgressStatus.IN_PROGRESS, ProgressStatus.NOT_STARTED],
        }
        return target_status in allowed_transitions.get(self, [])


@dataclass(frozen=True)
class NextAction:
    """次のアクション値オブジェクト"""

    title: str
    command: str
    time_estimation: "TimeEstimation"
    priority: str = "medium"

    def __post_init__(self) -> None:
        """値の検証"""
        if not self.title.strip():
            msg = "アクションタイトルは必須です"
            raise ValueError(msg)
        if not self.command.strip():
            msg = "実行コマンドは必須です"
            raise ValueError(msg)
        if self.priority not in ["high", "medium", "low"]:
            msg = "優先度は high, medium, low のいずれかである必要があります"
            raise ValueError(msg)

    def display_text(self) -> str:
        """表示用テキスト"""
        return f"{self.title} (所要時間: {self.time_estimation.display_text()})"

    @property
    def description(self) -> str:
        """アクション説明のエイリアス(互換性維持)"""
        return self.title

    @property
    def estimated_time(self) -> "TimeEstimation":
        """所要時間オブジェクトのエイリアス(互換性維持)"""
        return self.time_estimation



# File: src/mcp_servers/noveler/tools/check_readability_tool.py
# Purpose: Implement the "check_readability" MCP tool that evaluates manuscript
#          readability and returns findings and a score.
# Context: Part of the Noveler MCP tool suite. Uses domain readability
#          services and infrastructure path/cache services. No side effects at
#          import time; work happens during execute().
"""check_readability ツールの実装

SPEC-MCP-001: MCP Server Granular Microservice Architecture
"""
import hashlib
import re
import time
from pathlib import Path
from typing import Any

from mcp_servers.noveler.domain.entities.mcp_tool_base import MCPToolBase, ToolIssue, ToolRequest, ToolResponse
from noveler.domain.services.readability_analysis_service import ReadabilityAnalysisService
from noveler.domain.value_objects.readability_finding import ReadabilityFinding
from noveler.infrastructure.adapters.path_service_adapter import create_path_service
from noveler.infrastructure.caching.file_cache_service import get_file_cache_service
from noveler.infrastructure.logging.unified_logger import get_logger


class CheckReadabilityTool(MCPToolBase):
    """読みやすさチェックツール

    文章の読みやすさを評価し、改善点を提案する。
    - 文の長さ
    - 語彙の複雑さ
    - 読みやすさスコア
    """

    def __init__(self) -> None:
        super().__init__(
            tool_name="check_readability",
            tool_description="読みやすさチェック（文の長さ、難解語彙）"
        )
        self._analysis_service = ReadabilityAnalysisService()

    def get_input_schema(self) -> dict[str, Any]:
        """入力スキーマを返す"""
        schema = self._get_common_input_schema()

        # readability特有のパラメータを追加
        schema["properties"].update({
            "file_path": {
                "type": "string",
                "description": "直接ファイルパス指定（episode_numberより優先）",
            },
            "exclude_dialogue_lines": {
                "type": "boolean",
                "default": False,
                "description": "会話行（「…」/『…』）を文長チェックから除外する",
            },
            "check_aspects": {
                "type": "array",
                "items": {
                    "type": "string",
                    "enum": ["sentence_length", "vocabulary_complexity", "readability_score"]
                },
                "description": "チェック観点の絞り込み（省略時は全て）"
            }
        })

        return schema

    def execute(self, request: ToolRequest) -> ToolResponse:
        """読みやすさチェックを実行"""
        # 統一ロギングシステムを使用
        logger = get_logger(__name__)

        start_time = time.time()
        logger.info("読みやすさチェック開始: エピソード%s", request.episode_number)

        try:
            self._validate_request(request)

            # エピソードファイルの読み込み + パス/ハッシュ
            episode_content, file_path, file_hash = self._resolve_episode_content(request)
            if not episode_content:
                # 空のエピソードの場合は満点を返す
                logger.info("エピソード%s は空のため、満点を返却", request.episode_number)
                return self._create_response(True, 100.0, [], start_time)

            # チェック観点の決定
            check_aspects = self._get_check_aspects(request)
            logger.debug("チェック観点: %s", check_aspects)

            # 読みやすさ分析の実行
            analysis_result = self._analysis_service.analyze(
                episode_content, aspects=check_aspects,
                exclude_dialogue_lines=bool((request.additional_params or {}).get("exclude_dialogue_lines", False))
            )
            issues = self._convert_findings(
                analysis_result.issues,
                episode_content.split("\n"),
                file_path,
                file_hash,
            )
            score = analysis_result.score

            logger.info("読みやすさチェック完了: スコア=%.1f, 問題数=%d", score, len(issues))
            resp = self._create_response(True, score, issues, start_time)
            if file_path:
                resp.metadata["file_path"] = str(file_path)
            if file_hash:
                resp.metadata["file_hash"] = file_hash
            if analysis_result.average_sentence_length is not None:
                resp.metadata["average_sentence_length"] = analysis_result.average_sentence_length
            # PathServiceのフォールバック可視化
            self._apply_fallback_metadata(resp)
            return resp

        except FileNotFoundError:
            # より詳細なエラー情報を提供
            available_episodes = self._get_available_episodes()
            if available_episodes:
                error_msg = f"Episode {request.episode_number} not found. Available episodes: {', '.join(map(str, available_episodes))}"
            else:
                error_msg = f"Episode {request.episode_number} not found. No episodes are currently available in the manuscript directory."

            logger.warning(error_msg)
            return self._create_response(False, 0.0, [], start_time, error_msg)
        except Exception as e:
            error_msg = f"読みやすさチェックエラー: {e!s}"
            logger.exception(error_msg)
            return self._create_response(False, 0.0, [], start_time, error_msg)

    def _load_episode_content(self, episode_number: int, project_name: str | None) -> str:
        """エピソードファイルの内容を読み込み（基底クラスの共通実装を使用）"""
        return super()._load_episode_content(episode_number, project_name)

    def _get_check_aspects(self, request: ToolRequest) -> list[str]:
        """チェック観点を決定"""
        if request.additional_params and "check_aspects" in request.additional_params:
            return request.additional_params["check_aspects"]

        # デフォルトは全ての観点をチェック
        return ["sentence_length", "vocabulary_complexity", "readability_score"]

    def _get_available_episodes(self) -> list[int]:
        """利用可能なエピソード番号のリストを取得"""
        try:
            path_service = create_path_service()
            get_file_cache_service()
            manuscript_dir = path_service.get_manuscript_dir()

            # エピソードファイルのパターンで検索
            episode_numbers = []

            if manuscript_dir.exists():
                for file_path in manuscript_dir.glob("第*話_*.md"):
                    match = re.search(r"第(\d+)話_", file_path.name)
                    if match:
                        episode_numbers.append(int(match.group(1)))

            return sorted(episode_numbers)
        except Exception:
            return []

    def _convert_findings(
        self,
        findings: list[ReadabilityFinding],
        lines: list[str],
        file_path: Path | None,
        file_hash: str | None,
    ) -> list[ToolIssue]:
        reason_map = {
            "long_sentence": "LONG_SENTENCE",
            "complex_vocabulary": "COMPLEX_VOCABULARY",
            "high_average_sentence_length": "HIGH_AVG_SENTENCE_LENGTH",
        }

        tool_issues: list[ToolIssue] = []
        for finding in findings:
            ln_hash = None
            if finding.line_number and 1 <= finding.line_number <= len(lines):
                line_text = lines[finding.line_number - 1].strip()
                ln_hash = hashlib.sha256(line_text.encode("utf-8")).hexdigest()
            base = f"{file_hash}:{finding.issue_type}:{finding.line_number}:{ln_hash or ''}"
            iid = hashlib.sha256(base.encode("utf-8")).hexdigest()[:12]
            tool_issues.append(
                ToolIssue(
                    type=finding.issue_type,
                    severity=finding.severity,
                    message=finding.message,
                    line_number=finding.line_number,
                    suggestion=finding.suggestion,
                    file_path=str(file_path) if file_path else None,
                    line_hash=ln_hash,
                    reason_code=reason_map.get(finding.issue_type, finding.issue_type.upper()),
                    issue_id=f"readability-{iid}",
                )
                )
        return tool_issues

    def _resolve_episode_content(self, request: ToolRequest) -> tuple[str, Path | None, str | None]:
        ap = request.additional_params or {}
        file_path: Path | None = None

        if isinstance(ap.get("content"), str):
            content = ap.get("content") or ""
            if isinstance(ap.get("file_path"), str):
                candidate = Path(ap["file_path"]).expanduser()
                file_path = candidate if candidate.exists() else None
        elif isinstance(ap.get("file_path"), str):
            candidate = Path(ap["file_path"]).expanduser()
            file_path = candidate if candidate.exists() else None
            content = candidate.read_text(encoding="utf-8") if candidate.exists() else ""
        else:
            content = self._load_episode_content(request.episode_number, request.project_name)
            try:
                path_service = create_path_service()
                cache_service = get_file_cache_service()
                manuscript_dir = path_service.get_manuscript_dir()
                file_path = cache_service.get_episode_file_cached(manuscript_dir, request.episode_number)
                self._ps_collect_fallback(path_service)
            except Exception:
                file_path = None

        file_hash = hashlib.sha256(content.encode("utf-8")).hexdigest() if content else None
        return content, file_path, file_hash



#!/usr/bin/env python3
"""TDD新機能開発テンプレート
新機能開発時のRED-GREEN-REFACTORサイクル用テンプレート
"""

import sys
from pathlib import Path
from noveler.infrastructure.adapters.path_service_adapter import create_path_service
from typing import Any
from unittest.mock import patch

import pytest
import yaml

# テスト対象のモジュールをインポート
# # PathManager統一パス管理システムを使用(事前にパスを設定)
sys.path.insert(0, str(Path(__file__).parent.parent))  # scripts/を追加
from noveler.infrastructure.utils.path_manager import ensure_imports

ensure_imports()
# from your_module import YourClass


class TestNewFeature:
    """新機能のテストクラス

    TDDサイクル:
    1. RED: 失敗するテストを書く
    2. GREEN: 最小限の実装で通す
    3. REFACTOR: コードを改善する
    """

    @pytest.fixture
    def sample_data(self) -> dict[str, Any]:
        """テスト用のサンプルデータ"""
        return {
            "key1": "value1",
            "key2": "value2",
        }

    @pytest.fixture
    def temp_file(self, tmp_path: Path) -> Any:
        """テスト用の一時ファイル"""
        test_file = tmp_path / "test_file.yaml"
        test_data: dict[str, Any] = {"test": "data"}
        with Path(test_file).open("w", encoding="utf-8") as f:
            yaml.dump(test_data, f)
        return test_file

    @pytest.fixture
    def mock_config(self) -> dict[str, Any]:
        """設定のモック"""
        return {
            "project": {"name": "test_project"},
            "settings": {"debug": True},
        }

    # === RED Phase: 失敗するテストを書く ===

    def test_new_feature_basic_operation(self) -> None:
        """新機能の基本動作をテスト

        RED Phase:
        1. 期待する動作を明確に定義
        2. 実装前なので失敗することを確認
        """
        # Arrange: テスト環境の準備
        # feature = NewFeature()
        # input_data = sample_data

        # Act: 実際の処理実行
        # result = feature.process(input_data)
        # Assert: 期待する結果の確認
        # assert result == expected_result
        pytest.skip("実装前のテンプレート - 実際の機能に合わせて修正してください")

    def test_new_feature_error_handling(self) -> None:
        """エラーケースのテスト"""
        # Arrange: エラーが発生する状況を準備
        # feature = NewFeature()
        # invalid_input = None

        # Act & Assert: 適切なエラーが発生することを確認
        # with pytest.raises(ValueError, match="Invalid input"):
        #     feature.process(invalid_input)
        pytest.skip("実装前のテンプレート - エラーハンドリングを定義してください")

    def test_new_feature_boundary_valuetest(self) -> None:
        """境界値テストのテンプレート"""
        # 境界値の例:
        # - 空文字列、空リスト
        # - 最小値、最大値
        # - 制限値の±1
        pytest.skip("実装前のテンプレート - 境界値を定義してください")

    @pytest.mark.parametrize(
        ("input_value", "expected"),
        [
            ("test1", "result1"),
            ("test2", "result2"),
            ("test3", "result3"),
        ],
    )
    def test_new_feature_parameter_test(self) -> None:
        """複数のパターンを効率的にテスト"""
        # feature = NewFeature()
        # result = feature.process(input_value)
        # assert result == expected
        pytest.skip("実装前のテンプレート - パラメータを定義してください")

    # === GREEN Phase: 最小限の実装でテストを通す ===

    def test_new_feature_minimal_implementation(self) -> None:
        """最小限の実装でテストが通ることを確認"""
        # シンプルな実装でまずはテストを通す
        # リファクタリングは後で行う
        pytest.skip("GREEN Phase - 最小限の実装後に有効化してください")

    # === REFACTOR Phase: コードを改善する ===

    def test_new_feature_after_refactoring(self) -> None:
        """リファクタリング後もテストが通ることを確認"""
        # 実装を改善しても動作は変わらないことを確認
        pytest.skip("REFACTOR Phase - リファクタリング後に有効化してください")

    # === 統合テスト ===

    def test_new_feature_integration_test(self) -> None:
        """他のコンポーネントとの統合テスト"""
        # ファイルI/O、設定読み込み等を含む統合テスト
        pytest.skip("統合テスト - 他のコンポーネントとの連携を確認してください")

    # === モックを使用したテスト ===

    @patch("your_module.external_api_call")
    def test_new_feature_external_api_call(self, mock_api: object) -> None:
        """外部APIの呼び出しをモックしたテスト"""
        # 外部依存をモックして単体テストを実行
        mock_api.return_value = {"status": "success"}

        # feature = NewFeature()
        # result = feature.call_external_api()
        # assert result["status"] == "success"
        # mock_api.assert_called_once()
        pytest.skip("外部API呼び出しテスト - 実際のAPIに合わせて修正してください")

    # === パフォーマンステスト ===

    @pytest.mark.slow
    def test_new_feature_performance(self) -> None:
        """パフォーマンステスト(時間がかかる場合)"""

        # start_time = time.time()
        # feature = NewFeature()
        # result = feature.heavy_operation()
        # end_time = time.time()
        # assert end_time - start_time < 1.0  # 1秒以内
        pytest.skip("パフォーマンステスト - 実際の処理時間を測定してください")


# === テストユーティリティ ===


class TestHelper:
    """テスト用のヘルパークラス"""

    @staticmethod
    def create_sample_file(path: Path, content: dict) -> None:
        """テスト用のサンプルファイルを作成"""
        with Path(path).open("w", encoding="utf-8") as f:
            yaml.dump(content, f)

    @staticmethod
    def assert_file_exists(path: Path) -> None:
        """ファイルの存在を確認"""
        assert path.exists(), f"ファイルが存在しません: {path}"

    @staticmethod
    def assert_yaml_content(path: Path, expected: dict) -> None:
        """YAMLファイルの内容を確認"""
        with Path(path).open(encoding="utf-8") as f:
            actual = yaml.safe_load(f)
        assert actual == expected


# === カスタムフィクスチャ ===


@pytest.fixture
def project_structure(tmp_path: Path) -> Any:
    """プロジェクト構造をテスト用に作成"""
    project_root = tmp_path / "test_project"
    project_root.mkdir()

    # 必要なディレクトリを作成
    path_service = create_path_service(project_root)
    path_service.get_management_dir().mkdir()
    path_service.get_manuscript_dir().mkdir()

    # 設定ファイルを作成
    config_path = project_root / "プロジェクト設定.yaml"
    TestHelper.create_sample_file(config_path, {"project": {"name": "テストプロジェクト", "ncode": "N0000XX"}})

    return project_root


# === 使用方法 ===
#
# このテンプレートの使用方法:
# 1. ファイルをコピーして新機能用にリネーム
#    cp templates/tdd_template.py tests/test_your_new_feature.py
#
# 2. インポート文を修正
#    from your_module import YourClass
#
# 3. TDDサイクルを実行
#    a. RED: 失敗するテストを書く
#    b. GREEN: 最小限の実装で通す
#    c. REFACTOR: コードを改善する
#
# 4. テスト実行
#    pytest tests/test_your_new_feature.py -v
#
# 5. カバレッジ確認
#    pytest tests/test_your_new_feature.py --cov=your_module
#
# 実装例:
# - analyze_dropout_rate.py のテスト参照
# - check_basic_style.py のテスト参照
# - update_word_count.py のテスト参照



# File: src/noveler/domain/entities/ml_quality/__init__.py
# Purpose: ML quality entities module exports
# Context: Provides ML-enhanced quality evaluation entities

"""
ML Quality Entities.

This module provides entities for ML-enhanced quality evaluation:
- LearningModel: Trained ML model with metadata
- QualityFeedback: Evaluation feedback for learning
- CorrectionRecord: User correction record

Contract: SPEC-QUALITY-140 §3
"""

from noveler.domain.entities.ml_quality.learning_model import (
    LearningModel,
    ModelType,
)
from noveler.domain.entities.ml_quality.quality_feedback import (
    CorrectionRecord,
    EvaluationOutcome,
    FeedbackSource,
    QualityFeedback,
)

__all__ = [
    "CorrectionRecord",
    "EvaluationOutcome",
    "FeedbackSource",
    # Learning model
    "LearningModel",
    "ModelType",
    # Quality feedback
    "QualityFeedback",
]



#!/usr/bin/env python3
# File: tests/unit/infrastructure/test_deployment_system.py
# Purpose: Verify deployment domain and infrastructure behaviour across services and repositories.
# Context: Utilises shared path management to resolve project roots for deployment workflows.

from __future__ import annotations

import shutil
import sys
import tempfile
import unittest
from datetime import datetime, timezone
from pathlib import Path
from unittest.mock import MagicMock, Mock, patch

import pytest

# PathManager統一パス管理システムを使用(事前にパスを設定)
sys.path.insert(0, str(Path(__file__).parent.parent))  # scripts/を追加
from noveler.infrastructure.utils.path_manager import ensure_imports

ensure_imports()

from noveler.application.use_cases.deploy_scripts_use_case import DeployScriptsUseCase
from noveler.domain.deployment.entities import (
    Deployment,
    DeploymentMode,
    DeploymentStatus,
    DeploymentTarget,
    ScriptVersion,
)
from noveler.domain.deployment.value_objects import CommitHash, DeploymentConfig
from noveler.domain.value_objects.project_path import ProjectPath
from noveler.domain.value_objects.project_time import ProjectTimezone, project_datetime
from noveler.infrastructure.deployment.git_repository_impl import GitRepositoryImpl
from noveler.infrastructure.services.deployment_service import DeploymentService
from noveler.presentation.shared.shared_utilities import get_common_path_service


def resolve_project_path(raw_path: str | Path | None = None) -> ProjectPath:
    """共通PathServiceを介してプロジェクトパスを解決する"""

    path_service = get_common_path_service()
    if raw_path is None:
        resolved = path_service.project_root
    else:
        candidate = Path(raw_path)
        resolved = candidate if candidate.is_absolute() else path_service.project_root / candidate

    return ProjectPath(str(resolved))


# JSTタイムゾーン
JST = ProjectTimezone.jst().timezone

class TestDeploymentEntities(unittest.TestCase):
    """デプロイメントエンティティのテスト"""

    def test_create_deployment(self) -> None:
        """デプロイメントエンティティの作成"""
        # Arrange
        target = DeploymentTarget(
            project_path=resolve_project_path("/path/to/project"),
            project_name="テスト小説プロジェクト",
        )

        # Act
        deployment = Deployment(
            target=target,
            mode=DeploymentMode.PRODUCTION,
            source_commit=CommitHash("abcd123"),
            timestamp=datetime.now(JST),
        )

        # Assert
        assert deployment.status == DeploymentStatus.PENDING
        assert deployment.mode == DeploymentMode.PRODUCTION
        assert deployment.id is not None

    def test_deployment_state_transitions(self) -> None:
        """デプロイメントの状態遷移"""
        # Arrange
        deployment = self._create_test_deployment()

        # Act & Assert
        # 開始
        deployment.start()
        assert deployment.status == DeploymentStatus.IN_PROGRESS

        # 完了
        deployment.complete()
        assert deployment.status == DeploymentStatus.COMPLETED

        # 完了後は開始できない
        with pytest.raises(ValueError, match=".*"):
            deployment.start()

    def test_deployment_failure(self) -> None:
        """デプロイメント失敗の処理"""
        # Arrange
        deployment = self._create_test_deployment()
        deployment.start()

        # Act
        deployment.fail("ディスク容量不足")

        # Assert
        assert deployment.status == DeploymentStatus.FAILED
        assert deployment.error_message == "ディスク容量不足"
        assert deployment.completed_at is not None

    def test_deployment_validation(self) -> None:
        """デプロイメントの検証"""
        # Arrange
        deployment = self._create_test_deployment()

        # Act & Assert
        # 開発モードでは警告
        deployment.mode = DeploymentMode.DEVELOPMENT
        warnings = deployment.validate()
        assert "development" in warnings[0].lower()

        # 本番モードでは開発警告なし(ただし他の警告は出る可能性)
        deployment.mode = DeploymentMode.PRODUCTION
        warnings = deployment.validate()
        # 開発モード関連の警告がないことを確認
        assert not any("development" in w.lower() for w in warnings)

    def _create_test_deployment(self):
        """テスト用デプロイメント作成"""
        target = DeploymentTarget(
            project_path=resolve_project_path("/test/project"),
            project_name="Test Project",
        )

        return Deployment(
            target=target,
            mode=DeploymentMode.PRODUCTION,
            source_commit=CommitHash("abcd123"),
        )


class TestScriptVersion(unittest.TestCase):
    """スクリプトバージョンのテスト"""

    def test_version_comparison(self) -> None:
        """バージョンの比較"""
        # Arrange
        v1 = ScriptVersion(
            commit_hash=CommitHash("abc123"),
            timestamp=project_datetime(2024, 1, 1).datetime,
            deployed_by="user1",
        )

        v2 = ScriptVersion(
            commit_hash=CommitHash("def456"),
            timestamp=project_datetime(2024, 1, 2).datetime,
            deployed_by="user2",
        )

        # Act & Assert
        assert v2.is_newer_than(v1)
        assert not v1.is_newer_than(v2)

    def test_version_info_generation(self) -> None:
        """バージョン情報の生成"""
        # Arrange
        version = ScriptVersion(
            commit_hash=CommitHash("abc123"),
            timestamp=datetime(2024, 1, 1, 12, 0, tzinfo=timezone.utc),
            deployed_by="test_user",
            branch="main",
            mode=DeploymentMode.PRODUCTION,
        )

        # Act
        info = version.generate_version_info()

        # Assert
        assert "abc123" in info
        assert "2024-01-01" in info
        assert "PRODUCTION" in info
        assert "main" in info


class TestDeploymentService(unittest.TestCase):
    """デプロイメントサービスのテスト"""

    def setUp(self) -> None:
        self.git_repo = Mock()
        self.project_repo = Mock()
        self.service = DeploymentService(self.git_repo, self.project_repo)

    def test_check_uncommitted_changes(self) -> None:
        """未コミット変更のチェック"""
        # Arrange
        self.git_repo.has_uncommitted_changes.return_value = True
        self.git_repo.get_uncommitted_files.return_value = ["file1.py", "file2.py"]

        # Act
        has_changes, files = self.service.check_uncommitted_changes()

        # Assert
        assert has_changes
        assert len(files) == 2

    def test_find_deployable_projects(self) -> None:
        """デプロイ可能なプロジェクトの検索"""
        # Arrange
        self.project_repo.find_all_projects.return_value = [
            DeploymentTarget(resolve_project_path("/proj1"), "Project 1"),
            DeploymentTarget(resolve_project_path("/proj2"), "Project 2"),
        ]

        # Act
        projects = self.service.find_deployable_projects()

        # Assert
        assert len(projects) == 2
        assert projects[0].project_name == "Project 1"

    def test_validate_deployment_readiness(self) -> None:
        """デプロイ準備状態の検証"""
        # Arrange
        deployment = self._create_test_deployment()

        # 正常なケース
        self.git_repo.has_uncommitted_changes.return_value = False
        self.project_repo.project_exists.return_value = True

        # DeploymentTarget.validate()をモック化してエラーなしを返す
        with unittest.mock.patch.object(deployment.target, "validate", return_value=[]):
            # Act
            is_ready, issues = self.service.validate_deployment_readiness(deployment)

            # Assert
            assert is_ready
            assert len(issues) == 0

    def test_validate_deployment_with_issues(self) -> None:
        """問題があるデプロイメントの検証"""
        # Arrange
        deployment = self._create_test_deployment()
        deployment.mode = DeploymentMode.DEVELOPMENT

        self.git_repo.has_uncommitted_changes.return_value = True

        # Act
        is_ready, issues = self.service.validate_deployment_readiness(deployment)

        # Assert
        assert not is_ready
        assert len(issues) > 0
        assert any("uncommitted" in issue.lower() for issue in issues)

    def _create_test_deployment(self):
        target = DeploymentTarget(
            project_path=resolve_project_path("/test/project"),
            project_name="Test Project",
        )

        return Deployment(
            target=target,
            mode=DeploymentMode.PRODUCTION,
            source_commit=CommitHash("abcd123"),
        )


class TestDeployScriptsUseCase(unittest.TestCase):
    """デプロイスクリプトユースケースのテスト"""

    @pytest.fixture(autouse=True)
    def setup_temp_dir(self, tmp_path: Path) -> None:
        """テスト用の一時ディレクトリを設定"""
        self.temp_dir = str(tmp_path)

    def setUp(self) -> None:
        self.deployment_repo = Mock()
        self.deployment_service = Mock()
        self.version_service = Mock()

        self.use_case = DeployScriptsUseCase(
            self.deployment_repo,
            self.deployment_service,
            self.version_service,
        )

    def test_execute_single_deployment(self) -> None:
        """単一プロジェクトへのデプロイ"""
        # Arrange
        target = DeploymentTarget(
            project_path=resolve_project_path(self.temp_dir),
            project_name="Test Project",
        )

        # create_deploymentの戻り値を設定(PENDING状態で開始)
        deployment = Deployment(
            target=target,
            mode=DeploymentMode.PRODUCTION,
            source_commit=CommitHash("abc123"),
        )

        self.deployment_service.validate_deployment_readiness.return_value = (True, [])
        self.deployment_service.create_deployment.return_value = deployment
        self.version_service.get_current_commit.return_value = CommitHash("abc123")

        # デプロイメント処理の内部メソッドをモック化
        with unittest.mock.patch.object(self.use_case, "_perform_deployment") as mock_perform:
            mock_perform.return_value = None  # 成功を表す

            # Act
            from noveler.application.use_cases.deploy_scripts_use_case import DeployScriptsRequest

            request = DeployScriptsRequest(
                targets=[target],
                mode=DeploymentMode.PRODUCTION,
                force=False,
            )

            result = self.use_case.execute(request)

        # Assert
        assert result.success
        assert len(result.deployments) == 1
        assert result.deployments[0].status == DeploymentStatus.COMPLETED

    def test_execute_with_validation_failure(self) -> None:
        """検証失敗時のデプロイ中止"""
        # Arrange
        target = DeploymentTarget(
            project_path=resolve_project_path("/invalid/path"),
            project_name="Invalid Project",
        )

        # create_deploymentの戻り値を設定
        deployment = Deployment(
            target=target,
            mode=DeploymentMode.PRODUCTION,
            source_commit=CommitHash("abc123"),
        )

        self.deployment_service.validate_deployment_readiness.return_value = (
            False,
            ["Project directory does not exist"],
        )

        self.deployment_service.create_deployment.return_value = deployment

        # Act
        result = self.use_case.execute(
            targets=[target],
            mode=DeploymentMode.PRODUCTION,
            force=False,
        )

        # Assert
        assert not result.success
        assert "validation" in result.error_message.lower()

    def test_rollback_on_failure(self) -> None:
        """失敗時のロールバック"""
        # Arrange
        target = DeploymentTarget(
            project_path=resolve_project_path(self.temp_dir),
            project_name="Test Project",
        )

        # create_deploymentの戻り値を設定
        deployment = Deployment(
            target=target,
            mode=DeploymentMode.PRODUCTION,
            source_commit=CommitHash("abc123"),
        )

        self.deployment_service.validate_deployment_readiness.return_value = (True, [])
        self.deployment_service.create_deployment.return_value = deployment

        # _perform_deploymentでエラーを発生させる
        with unittest.mock.patch.object(self.use_case, "_perform_deployment") as mock_perform:
            mock_perform.side_effect = Exception("Disk full")

            # Act
            result = self.use_case.execute(
                targets=[target],
                mode=DeploymentMode.PRODUCTION,
                force=False,
            )

        # Assert
        assert not result.success
        # デプロイメントが失敗状態になることを確認
        assert deployment.status == DeploymentStatus.FAILED


class TestDeploymentConfig(unittest.TestCase):
    """デプロイメント設定のテスト"""

    def test_default_config(self) -> None:
        """デフォルト設定の確認"""
        # Act
        config = DeploymentConfig()

        # Assert
        assert config.scripts_directory_name == ".novel-scripts"
        assert config.create_backup
        assert config.backup_retention_days == 7

    def test_config_validation(self) -> None:
        """設定の検証"""
        # Arrange
        config = DeploymentConfig(
            scripts_directory_name="invalid/name",
            backup_retention_days=-1,
        )

        # Act
        errors = config.validate()

        # Assert
        assert len(errors) > 0
        assert any("directory name" in e.lower() for e in errors)


class TestIntegration(unittest.TestCase):
    """統合テスト"""

    @patch("subprocess.run")
    @patch("os.path.exists")
    @patch("noveler.infrastructure.deployment.git_deployment_synchronizer.GitDeploymentSynchronizer")
    def test_full_deployment_flow(self, mock_synchronizer_class: object, mock_exists: object, mock_run: object) -> None:
        """完全なデプロイメントフロー(TDD準拠修正版)"""
        # Arrange - TDDの原則に従って明確なテストデータ作成
        mock_exists.return_value = True
        mock_run.return_value = MagicMock(returncode=0, stdout="deployment success")

        # TDD: GitDeploymentSynchronizerのモック設定
        mock_synchronizer = MagicMock()
        mock_synchronizer.create_backup.return_value = "backup_20240714_120000"
        mock_synchronizer.sync_to_latest.return_value = True
        mock_synchronizer.verify_deployment.return_value = True
        mock_synchronizer_class.return_value = mock_synchronizer

        temp_dir = Path(tempfile.mkdtemp())
        self.addCleanup(lambda: shutil.rmtree(temp_dir, ignore_errors=True))
        # プロジェクト構造を作成
        project_dir = temp_dir / "test_project"
        project_dir.mkdir()
        (project_dir / "プロジェクト設定.yaml").touch()

        # DDD: エンティティとサービスを適切にモック
        deployment_target = DeploymentTarget(
            resolve_project_path(project_dir),
            "Test Project",
        )

        # リポジトリとサービスを初期化
        git_repo = Mock(spec=GitRepositoryImpl)
        git_repo.get_current_commit.return_value = CommitHash("abc123def")
        git_repo.has_uncommitted_changes.return_value = False

        deployment_repo = Mock()
        deployment_repo.save.return_value = True

        project_repo = Mock()
        project_repo.find_all_projects.return_value = [deployment_target]

        # DDD: ドメインサービスのモック
        deployment_service = Mock()
        deployment_service.find_deployable_projects.return_value = [deployment_target]

        # TDD: ドメインサービスが作成するデプロイメントエンティティ(PENDING状態)
        pending_deployment = Deployment(
            target=deployment_target,
            mode=DeploymentMode.PRODUCTION,
            source_commit=CommitHash("abc123def"),
        )

        # 注意: デプロイメントはPENDING状態でcreate_deploymentから返され、
        # Use Case内で状態遷移が制御される

        # TDD: 実際のサービス仕様に合わせてcreate_deploymentメソッドを使用
        deployment_service.create_deployment.return_value = pending_deployment
        deployment_service.validate_deployment_readiness.return_value = (True, [])

        version_service = Mock()
        version_service.get_current_commit.return_value = CommitHash("abc123def")
        version_service.get_current_branch.return_value = "main"

        use_case = DeployScriptsUseCase(
            deployment_repo,
            deployment_service,
            version_service,
        )

        # Act
        result = use_case.execute(
            targets=None,  # すべてのプロジェクト
            mode=DeploymentMode.PRODUCTION,
            force=True,  # テストなので強制実行
        )

        # Assert - TDD: 期待される結果を明確に検証
        assert result.success, f"Expected success but got: {result.error_message}"
        assert len(result.deployments) > 0, "Expected at least one deployment"
        # TDD: Use Caseでデプロイメント処理が正常に実行された場合の状態確認
        # 注意: pending_deploymentの状態はUse Case内で変更される

        # DDD: ドメインサービスが適切に呼び出されたことを検証
        deployment_service.find_deployable_projects.assert_called_once()
        # TDD: Use Caseが内部で_deploy_to_targetを呼び出すため、deployment_serviceのcreate_deploymentが呼ばれることを確認


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
"""
プロンプト生成エンティティ

A24ガイドベースのプロンプト生成処理を管理するドメインエンティティ。
DDD設計に基づき、プロンプト生成のビジネスルールと状態を管理する。
"""

from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import Any
from uuid import UUID, uuid4

from noveler.domain.value_objects.project_time import project_now


class A24Stage(Enum):
    """A24ガイドの4段階プロセス"""

    SKELETON = "骨格構築"  # Stage 1: 基本情報・概要定義
    THREE_ACT = "三幕構成設計"  # Stage 2: 物語構造設計
    SCENE_DETAIL = "シーン肉付け"  # Stage 3: シーン詳細化
    TECH_INTEGRATION = "技術伏線統合"  # Stage 4: 技術要素・伏線統合


class ContextElementType(Enum):
    """コンテキスト要素の種類"""

    FORESHADOWING = "伏線要素"
    IMPORTANT_SCENE = "重要シーン"
    CHAPTER_CONNECTION = "章間連携"
    TECHNICAL_ELEMENT = "技術要素"


class OptimizationTarget(Enum):
    """最適化ターゲット"""

    CLAUDE_CODE = "Claude Code"
    CLAUDE_WEB = "Claude Web"
    CHATGPT = "ChatGPT"
    GENERIC = "汎用"


@dataclass(frozen=True)
class ContextElement:
    """コンテキスト要素バリューオブジェクト

    プロンプト生成に使用される追加情報の単位。
    伏線、重要シーン、技術要素など様々なコンテキストを統一的に扱う。
    """

    element_type: ContextElementType
    content: str
    priority: float  # 0.0-1.0, 1.0が最高優先度
    integration_stage: A24Stage
    source_file: str | None = None
    metadata: dict[str, Any] = field(default_factory=dict)

    def __post_init__(self) -> None:
        """データ妥当性検証"""
        if not 0.0 <= self.priority <= 1.0:
            msg = f"優先度は0.0-1.0の範囲である必要があります: {self.priority}"
            raise ValueError(msg)

        if not self.content.strip():
            msg = "コンテンツは空にできません"
            raise ValueError(msg)


@dataclass(frozen=True)
class A24StagePrompt:
    """A24段階別プロンプトバリューオブジェクト

    A24ガイドの各段階に対応するプロンプト構成要素。
    段階ごとの指示、検証基準、期待される出力を定義する。
    """

    stage: A24Stage
    instructions: list[str]
    validation_criteria: list[str]
    expected_output_format: str
    context_integration_points: list[str] = field(default_factory=list)

    def __post_init__(self) -> None:
        """データ妥当性検証"""
        if not self.instructions:
            msg = "指示リストは空にできません"
            raise ValueError(msg)

        if not self.expected_output_format.strip():
            msg = "期待出力フォーマットは空にできません"
            raise ValueError(msg)


@dataclass
class PromptGenerationSession:
    """プロンプト生成セッション エンティティ

    1回のプロンプト生成処理の状態と履歴を管理するエンティティ。
    生成過程の追跡、品質管理、エラーハンドリングを担当する。
    """

    session_id: UUID = field(default_factory=uuid4)
    episode_number: int = 0
    project_name: str = ""
    created_at: datetime = field(default_factory=lambda: project_now().datetime)

    # 生成設定
    context_level: str = "基本"  # 基本|拡張|完全
    optimization_target: OptimizationTarget = OptimizationTarget.CLAUDE_CODE
    include_foreshadowing: bool = True
    include_important_scenes: bool = True

    # 生成状態
    current_stage: A24Stage | None = None
    completed_stages: list[A24Stage] = field(default_factory=list)
    generated_content: dict[A24Stage, str] = field(default_factory=dict)
    integrated_context: list[ContextElement] = field(default_factory=list)

    # 生成結果
    final_prompt: str = ""
    token_estimate: int = 0
    generation_time_ms: int = 0

    # エラー管理
    errors: list[str] = field(default_factory=list)
    warnings: list[str] = field(default_factory=list)

    def start_generation(self, episode_number: int, project_name: str) -> None:
        """プロンプト生成開始"""
        if self.is_completed():
            msg = "既に完了したセッションです"
            raise ValueError(msg)

        self.episode_number = episode_number
        self.project_name = project_name
        self.current_stage = A24Stage.SKELETON
        self.created_at = project_now().datetime

    def advance_to_stage(self, stage: A24Stage) -> None:
        """指定段階に進行"""
        if self.current_stage and self.current_stage not in self.completed_stages:
            self.completed_stages.append(self.current_stage)

        self.current_stage = stage

    def complete_current_stage(self, generated_content: str) -> None:
        """現在段階の完了"""
        if not self.current_stage:
            msg = "アクティブな段階がありません"
            raise ValueError(msg)

        self.generated_content[self.current_stage] = generated_content
        if self.current_stage not in self.completed_stages:
            self.completed_stages.append(self.current_stage)

    def add_context_element(self, element: ContextElement) -> None:
        """コンテキスト要素追加"""
        self.integrated_context.append(element)

    def finalize_prompt(self, final_prompt: str, token_estimate: int, generation_time_ms: int) -> None:
        """プロンプト生成完了"""
        self.final_prompt = final_prompt
        self.token_estimate = token_estimate
        self.generation_time_ms = generation_time_ms

        # 全段階完了確認
        all_stages = list(A24Stage)
        missing_stages = [stage for stage in all_stages if stage not in self.completed_stages]
        if missing_stages:
            stage_names = [stage.value for stage in missing_stages]
            self.warnings.append(f"未完了段階があります: {', '.join(stage_names)}")

    def add_error(self, error_message: str) -> None:
        """エラー追加"""
        self.errors.append(f"{project_now().datetime.isoformat()}: {error_message}")

    def add_warning(self, warning_message: str) -> None:
        """警告追加"""
        self.warnings.append(f"{project_now().datetime.isoformat()}: {warning_message}")

    def is_completed(self) -> bool:
        """生成完了判定"""
        return len(self.final_prompt) > 0 and len(self.errors) == 0

    def is_success(self) -> bool:
        """生成成功判定"""
        return self.is_completed() and len(self.completed_stages) >= 3  # 最低3段階は完了必要

    def get_completion_rate(self) -> float:
        """完了率取得 (0.0-1.0)"""
        total_stages = len(list(A24Stage))
        completed_count = len(self.completed_stages)
        return min(1.0, completed_count / total_stages)

    def get_context_summary(self) -> dict[str, int]:
        """コンテキストサマリ取得"""
        summary = {}
        for element_type in ContextElementType:
            count = len([e for e in self.integrated_context if e.element_type == element_type])
            summary[element_type.value] = count
        return summary

    def get_session_statistics(self) -> dict[str, Any]:
        """セッション統計取得"""
        return {
            "session_id": str(self.session_id),
            "episode_number": self.episode_number,
            "project_name": self.project_name,
            "completion_rate": self.get_completion_rate(),
            "context_summary": self.get_context_summary(),
            "token_estimate": self.token_estimate,
            "generation_time_ms": self.generation_time_ms,
            "error_count": len(self.errors),
            "warning_count": len(self.warnings),
            "success": self.is_success(),
        }



#!/usr/bin/env python3
"""高度文章リズム分析エンジン

手動Claude Code分析レベルの文章リズム・テンポ分析を実現する
ドメインサービス。文の長短バランス、読点配置、呼吸感を定量化。
"""

import re
from dataclasses import dataclass
from statistics import mean, stdev

from noveler.domain.value_objects.improvement_suggestion import ImprovementSuggestion


@dataclass
class RhythmAnalysisResult:
    """文章リズム分析結果"""

    overall_score: float
    short_sentence_ratio: float  # 短文比率
    medium_sentence_ratio: float  # 中文比率
    long_sentence_ratio: float  # 長文比率
    punctuation_density: float  # 読点密度
    rhythm_variation_score: float  # リズム変化スコア
    tempo_consistency: float  # テンポ一貫性
    breathing_points_score: float  # 呼吸点スコア
    problematic_lines: list[int]  # 問題のある行番号
    specific_suggestions: list[ImprovementSuggestion]

    def get_detailed_feedback(self) -> str:
        """詳細フィードバックを生成"""
        feedback_parts = []

        # 文長バランス評価
        if self.short_sentence_ratio < 0.2:
            feedback_parts.append("短文が不足しています。テンポを生むため、簡潔な文を追加してください。")
        elif self.short_sentence_ratio > 0.6:
            feedback_parts.append("短文が過多です。内容の深みを出すため、長めの文も織り交ぜてください。")

        # リズム変化評価
        if self.rhythm_variation_score < 60:
            feedback_parts.append("文章のリズムが単調です。文の長さに変化をつけて読みやすさを向上させてください。")

        # 読点密度評価
        if self.punctuation_density < 0.1:
            feedback_parts.append("読点が不足しています。適切な区切りで読者の理解を助けてください。")
        elif self.punctuation_density > 0.4:
            feedback_parts.append("読点が多すぎます。文の流れを阻害しない程度に調整してください。")

        return " ".join(feedback_parts) if feedback_parts else "文章リズムは良好です。"


class AdvancedRhythmAnalyzer:
    """高度文章リズム分析エンジン

    手動分析レベルの詳細なリズム・テンポ分析を実行。
    文の長短、読点配置、呼吸感を総合的に評価。
    """

    def __init__(self) -> None:
        """高度リズム分析エンジン初期化"""
        self._sentence_length_thresholds = {
            "short": 15,  # 15文字以下は短文
            "medium": 40,  # 16-40文字は中文
            "long": 40,  # 41文字以上は長文
        }
        self._ideal_ratios = {
            "short": 0.3,  # 理想短文比率
            "medium": 0.5,  # 理想中文比率
            "long": 0.2,  # 理想長文比率
        }

    def analyze_rhythm(self, content: str) -> RhythmAnalysisResult:
        """文章リズムの詳細分析

        Args:
            content: 分析対象テキスト

        Returns:
            RhythmAnalysisResult: リズム分析結果
        """
        sentences = self._extract_sentences(content)
        if len(sentences) < 3:
            return self._create_insufficient_data_result()

        # 文長分析
        sentence_lengths = [len(s.strip()) for s in sentences if s.strip()]
        length_ratios = self._calculate_length_ratios(sentence_lengths)

        # 読点密度分析
        punctuation_density = self._calculate_punctuation_density(content)

        # リズム変化分析
        rhythm_variation = self._analyze_rhythm_variation(sentence_lengths)

        # テンポ一貫性分析
        tempo_consistency = self._analyze_tempo_consistency(sentences)

        # 呼吸点分析
        breathing_score = self._analyze_breathing_points(content)

        # 問題箇所特定
        problematic_lines = self._identify_problematic_lines(content, sentences)

        # 具体的改善提案生成
        suggestions = self._generate_rhythm_suggestions(length_ratios, punctuation_density, rhythm_variation)

        # 総合スコア計算
        overall_score = self._calculate_overall_rhythm_score(
            length_ratios, punctuation_density, rhythm_variation, tempo_consistency, breathing_score
        )

        return RhythmAnalysisResult(
            overall_score=overall_score,
            short_sentence_ratio=length_ratios["short"],
            medium_sentence_ratio=length_ratios["medium"],
            long_sentence_ratio=length_ratios["long"],
            punctuation_density=punctuation_density,
            rhythm_variation_score=rhythm_variation,
            tempo_consistency=tempo_consistency,
            breathing_points_score=breathing_score,
            problematic_lines=problematic_lines,
            specific_suggestions=suggestions,
        )

    def _extract_sentences(self, content: str) -> list[str]:
        """文の抽出"""
        # 日本語の文境界を考慮した分割
        sentences = re.split(r"[。！？](?![」』）])", content)
        return [s.strip() for s in sentences if s.strip()]

    def _calculate_length_ratios(self, sentence_lengths: list[int]) -> dict[str, float]:
        """文長比率の計算"""
        if not sentence_lengths:
            return {"short": 0.0, "medium": 0.0, "long": 0.0}

        short_count = sum(1 for length in sentence_lengths if length <= self._sentence_length_thresholds["short"])
        long_count = sum(1 for length in sentence_lengths if length > self._sentence_length_thresholds["long"])
        medium_count = len(sentence_lengths) - short_count - long_count

        total = len(sentence_lengths)
        return {"short": short_count / total, "medium": medium_count / total, "long": long_count / total}

    def _calculate_punctuation_density(self, content: str) -> float:
        """読点密度の計算"""
        comma_count = content.count("、")
        char_count = len(content)
        return comma_count / char_count if char_count > 0 else 0.0

    def _analyze_rhythm_variation(self, sentence_lengths: list[int]) -> float:
        """リズム変化の分析"""
        if len(sentence_lengths) < 3:
            return 50.0

        # 連続する文の長さの変化を分析
        length_changes = []
        for i in range(1, len(sentence_lengths)):
            change = abs(sentence_lengths[i] - sentence_lengths[i - 1])
            length_changes.append(change)

        if not length_changes:
            return 50.0

        # 変化の標準偏差が大きいほど良いリズム
        avg_change = mean(length_changes)
        return min(100.0, avg_change * 2)  # スケーリング

    def _analyze_tempo_consistency(self, sentences: list[str]) -> float:
        """テンポ一貫性の分析"""
        if len(sentences) < 3:
            return 80.0

        # 各文の「テンポ感」を分析
        tempo_scores = []
        for sentence in sentences:
            # 短文・会話文は高テンポ
            tempo = 50.0
            if len(sentence) < 20:
                tempo += 20
            if "「" in sentence:
                tempo += 15
            if any(word in sentence for word in ["！", "？", "だ", "した"]):
                tempo += 10

            tempo_scores.append(min(100.0, tempo))

        # テンポの標準偏差が小さいほど一貫性が高い
        if len(tempo_scores) > 1:
            tempo_stdev = stdev(tempo_scores)
            consistency = max(0.0, 100.0 - tempo_stdev * 2)
        else:
            consistency = 80.0

        return consistency

    def _analyze_breathing_points(self, content: str) -> float:
        """呼吸点（自然な区切り）の分析"""
        lines = content.split("\n")
        paragraph_count = len([line for line in lines if line.strip()])

        if paragraph_count == 0:
            return 0.0

        # 段落区切り、読点、接続詞による呼吸点を評価
        breathing_points = 0

        # 段落区切りによる呼吸点
        empty_lines = content.count("\n\n")
        breathing_points += empty_lines * 10

        # 読点による呼吸点
        commas = content.count("、")
        breathing_points += commas * 2

        # 接続詞による呼吸点
        conjunctions = len(re.findall(r"(しかし|そして|だから|それで|ところが)", content))
        breathing_points += conjunctions * 5

        # 長さに対する適切性を評価
        content_length = len(content)
        if content_length > 0:
            breathing_density = breathing_points / (content_length / 100)
            return min(100.0, breathing_density * 20)

        return 50.0

    def _identify_problematic_lines(self, content: str, sentences: list[str]) -> list[int]:
        """問題のある行を特定"""
        lines = content.split("\n")
        problematic_lines = []

        for i, line in enumerate(lines, 1):
            line = line.strip()
            if not line:
                continue

            # 極端に長い文（80文字以上）
            if len(line) > 80:
                problematic_lines.append(i)

            # 読点が多すぎる文（8個以上）
            if line.count("、") > 8:
                problematic_lines.append(i)

            # 単調な文末パターン
            if line.endswith(("だった。", "だった。", "だった。")):
                # 前後の行も同じパターンかチェック
                prev_line = lines[i - 2].strip() if i > 1 else ""
                next_line = lines[i].strip() if i < len(lines) else ""

                if prev_line.endswith("だった。") or next_line.endswith("だった。"):
                    problematic_lines.append(i)

        return problematic_lines

    def _generate_rhythm_suggestions(
        self, length_ratios: dict[str, float], punctuation_density: float, rhythm_variation: float
    ) -> list[ImprovementSuggestion]:
        """リズム改善提案の生成"""
        suggestions = []

        # 文長バランス提案
        if length_ratios["short"] < 0.2:
            suggestions.append(
                ImprovementSuggestion.create(
                    content="テンポ向上のため、短文（15文字以下）を増やしてください。例：「彼は振り返った。」",
                    suggestion_type="style",
                )
            )

        if length_ratios["long"] > 0.4:
            suggestions.append(
                ImprovementSuggestion.create(
                    content="読みやすさのため、長文を短く分割してください。読点で区切られる部分を独立した文にできます。",
                    suggestion_type="style",
                )
            )

        # 読点密度提案
        if punctuation_density < 0.1:
            suggestions.append(
                ImprovementSuggestion.create(
                    content="文の区切りを明確にするため、適切な位置に読点「、」を追加してください。",
                    suggestion_type="style",
                )
            )

        # リズム変化提案
        if rhythm_variation < 60:
            suggestions.append(
                ImprovementSuggestion.create(
                    content="文章のリズムに変化をつけてください。短文・中文・長文を効果的に組み合わせることで読者を飽きさせません。",
                    suggestion_type="enhancement",
                )
            )

        return suggestions

    def _calculate_overall_rhythm_score(
        self,
        length_ratios: dict[str, float],
        punctuation_density: float,
        rhythm_variation: float,
        tempo_consistency: float,
        breathing_score: float,
    ) -> float:
        """総合リズムスコアの計算"""

        # 文長バランス評価（30%重み）
        balance_score = 100.0
        for ratio_type, actual_ratio in length_ratios.items():
            ideal_ratio = self._ideal_ratios[ratio_type]
            deviation = abs(actual_ratio - ideal_ratio)
            balance_score -= deviation * 100  # 偏差をペナライズ

        balance_contribution = max(0.0, balance_score) * 0.3

        # 読点密度評価（20%重み）
        ideal_punctuation = 0.15  # 理想的な読点密度
        punctuation_deviation = abs(punctuation_density - ideal_punctuation)
        punctuation_contribution = max(0.0, 100.0 - punctuation_deviation * 200) * 0.2

        # リズム変化評価（25%重み）
        rhythm_contribution = rhythm_variation * 0.25

        # テンポ一貫性評価（15%重み）
        tempo_contribution = tempo_consistency * 0.15

        # 呼吸点評価（10%重み）
        breathing_contribution = breathing_score * 0.1

        final_score = (
            balance_contribution
            + punctuation_contribution
            + rhythm_contribution
            + tempo_contribution
            + breathing_contribution
        )

        return max(0.0, min(100.0, final_score))

    def _create_insufficient_data_result(self) -> RhythmAnalysisResult:
        """データ不足時の結果作成"""
        return RhythmAnalysisResult(
            overall_score=70.0,
            short_sentence_ratio=0.0,
            medium_sentence_ratio=0.0,
            long_sentence_ratio=0.0,
            punctuation_density=0.0,
            rhythm_variation_score=50.0,
            tempo_consistency=70.0,
            breathing_points_score=50.0,
            problematic_lines=[],
            specific_suggestions=[
                ImprovementSuggestion.create(
                    content="分析に十分なテキスト量がありません。より多くの文章でリズム分析の精度が向上します。",
                    suggestion_type="enhancement",
                )
            ],
        )



"""BackupUseCase の B20 版ユニットテスト"""

from __future__ import annotations

from pathlib import Path
from types import SimpleNamespace
from unittest.mock import Mock

import pytest

from noveler.application.use_cases.backup_use_case import BackupRequest, BackupResponse, BackupUseCase


@pytest.fixture
def backup_use_case(monkeypatch, tmp_path: Path) -> SimpleNamespace:
    logger = Mock()

    project_repository = Mock()
    project_repository.find_by_name.return_value = SimpleNamespace(name="テストプロジェクト")

    backup_repository = Mock()
    backup_repository.backup_file.return_value = True

    transaction = Mock()
    transaction.__enter__ = Mock(return_value=transaction)
    transaction.__exit__ = Mock(return_value=None)

    unit_of_work = Mock()
    unit_of_work.project_repository = project_repository
    unit_of_work.backup_repository = backup_repository
    unit_of_work.transaction.return_value = transaction

    path_service = Mock()
    path_service.get_temp_dir.return_value = tmp_path
    path_service.project_root = tmp_path
    path_service.get_manuscript_dir.return_value = tmp_path

    monkeypatch.setattr(
        "noveler.application.use_cases.backup_use_case.create_path_service",
        lambda: path_service,
    )

    use_case = BackupUseCase(logger_service=logger, unit_of_work=unit_of_work)
    return SimpleNamespace(
        use_case=use_case,
        project_repository=project_repository,
        backup_repository=backup_repository,
        path_service=path_service,
    )


def test_backup_response_helpers() -> None:
    response = BackupResponse(
        success=True,
        backup_path=Path("/tmp/example"),
        backup_size=1.5,
        file_count=3,
        backup_metadata={"duration": 2.0, "message": "done"},
    )

    assert response.total_size == int(1.5 * 1024 * 1024)
    assert response.duration == 2.0
    assert response.message == "done"


@pytest.mark.asyncio
async def test_execute_full_backup_success(backup_use_case: SimpleNamespace, tmp_path: Path) -> None:
    (tmp_path / "プロジェクト設定.yaml").write_text("name: テスト")
    (tmp_path / "novel.md").write_text("content")

    request = BackupRequest(project_name="テストプロジェクト", backup_type="full")

    response = await backup_use_case.use_case.execute(request)

    assert response.success is True
    assert response.backup_path is not None
    assert response.file_count >= 0
    assert response.backup_metadata is not None
    assert response.backup_metadata["backup_type"] == "full"


@pytest.mark.asyncio
async def test_execute_returns_error_when_project_missing(backup_use_case: SimpleNamespace) -> None:
    backup_use_case.project_repository.find_by_name.return_value = None

    request = BackupRequest(project_name="存在しないプロジェクト")
    response = await backup_use_case.use_case.execute(request)

    assert response.success is False
    assert "見つかりません" in response.error_message


@pytest.mark.asyncio
async def test_execute_incremental_backup(backup_use_case: SimpleNamespace) -> None:
    request = BackupRequest(project_name="テストプロジェクト", backup_type="incremental")

    response = await backup_use_case.use_case.execute(request)

    assert response.success in {True, False}
    assert response.backup_path is not None or response.error_message is not None
